/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 29.08.00
*/


#include <qstring.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qapplication.h>

#include <QXProgressCylinder.hh>

#include <QScrunitizer.hh>

#include "Scrunitizer.xpm"
#include "FailureSign.xpm"
#include "ErrorSign.xpm"
#include "PUnknown.xpm"
#include "PSetUp.xpm"
#include "PExec.xpm"
#include "PPremTDown.xpm"
#include "PTearDown.xpm"
#include "ScrunitizerHeader.xpm"



class ExRecItem : public QListViewItem {

	public:
		static const QString _strA;
		static const QString _strB;
		static const QString _strC;
		static const QString _strD;
		static const QString _strE;
		static const QString* _strTab[5];

		static QPixmap*	_failureSign;
		static QPixmap*	_errorSign;
		static QPixmap*	_phaseIcon[5];

		static int clients;

		static void acquire ();
		static void release ();

	private:
		Scrunitizer::TestExceptionRecord& _exceptionRecord;

	public:
		ExRecItem (QListView* parent, Scrunitizer::TestExceptionRecord& er, int no);

	public:
		QString key ( int column, bool ascending ) const;
		
};

const QString ExRecItem::_strA ("A");
const QString ExRecItem::_strB ("B");
const QString ExRecItem::_strC ("C");
const QString ExRecItem::_strD ("D");
const QString ExRecItem::_strE ("E");
const QString* ExRecItem::_strTab[5] = {&_strA, &_strB, &_strC, &_strD, &_strE};

QPixmap* ExRecItem::_failureSign;
QPixmap* ExRecItem::_errorSign;
QPixmap* ExRecItem::_phaseIcon[5];
int ExRecItem::clients = 0;


void ExRecItem::acquire () {
	if (clients++ == 0) {
		_failureSign = new QPixmap (const_cast<const char**>(FailureSign_xpm));
		_errorSign   = new QPixmap (const_cast<const char**>(ErrorSign_xpm));

		_phaseIcon[0] = new QPixmap (const_cast<const char**>(PUnknown_xpm));
		_phaseIcon[1] = new QPixmap (const_cast<const char**>(PSetUp_xpm));
		_phaseIcon[2] = new QPixmap (const_cast<const char**>(PExec_xpm));
		_phaseIcon[3] = new QPixmap (const_cast<const char**>(PPremTDown_xpm));
		_phaseIcon[4] = new QPixmap (const_cast<const char**>(PTearDown_xpm));
	}
}


void ExRecItem::release () {
	if (--clients == 0) {
		delete _failureSign;
		delete _errorSign;

		int i;
		for (i = 0; i < 5; i++) {
			delete _phaseIcon[i];
		}
	}
}


ExRecItem::ExRecItem (QListView* parent, Scrunitizer::TestExceptionRecord& er, int no) 
 : QListViewItem (parent), _exceptionRecord (er) {
	QString nstr;
	nstr.sprintf ("%5d", no);
	
 	setText   (0, nstr);
	setPixmap (1, (er.isErrorRecord ()) ? *_errorSign : *_failureSign);
 	setText   (2, ((er.getOrigin ())->getName ()).c_str ());
	setPixmap (3, *(_phaseIcon[(int)er.getPhase ()]));
	setText   (4, ((er.getException ())->getMessage ()).c_str ());
}


QString ExRecItem::key (int column, bool ascending) const {
	if (column == 1) {
		return (_exceptionRecord.isErrorRecord ()) ? _strA : _strB;
	}
	if (column == 3) {
		return *_strTab[(int)_exceptionRecord.getPhase ()];
	}
	return text (column);
}



QScrunitizer::QScrunitizer (
) : QWidget (0, "main"), _tests (0), _testing (false), _manager (0) {

	QLabel* label;

	QGridLayout* layout = new QGridLayout (this, 2, 1, 0, 0);
	layout->setRowStretch (1, 1);

	QColor bgBlue (49, 47, 144);
	QColor white (255, 255, 255);
	QPalette bgBluePalette (bgBlue, bgBlue);
	bgBluePalette.setColor (QColorGroup::Foreground, white);
	bgBluePalette.setColor (QColorGroup::Text, white);
	bgBluePalette.setColor (QColorGroup::Light, bgBlue.light ());
	bgBluePalette.setColor (QColorGroup::Highlight,  bgBlue.light ());
	bgBluePalette.setColor (QColorGroup::Dark, bgBlue.dark ());
	bgBluePalette.setColor (QColorGroup::Shadow, bgBlue.dark ());

	QFrame* stateFrame = new QFrame (this, "state");
	stateFrame->setFrameStyle (QFrame::Panel | QFrame::Raised);
	stateFrame->setPalette (bgBluePalette);
	QVBoxLayout* stateLayout = new QVBoxLayout (stateFrame, 1);

	_progressIndicator = new QXProgressCylinder (stateFrame, "progress");
	_progressIndicator->setMinimumSize (QSize (400, 32));
	_progressIndicator->setPalette (bgBluePalette);
	_progressIndicator->setMargin (6);
	stateLayout->addWidget (_progressIndicator);	

	QHBoxLayout* paneLayout = new QHBoxLayout (6);

	QPixmap scrunitizerImg (const_cast<const char**>(ScrunitizerHeader));
	label = new QLabel (stateFrame);
	label->setPixmap (scrunitizerImg);
	label->setMinimumSize (QSize (440, 150));
	paneLayout->addWidget (label);

	QFrame* infoFrame = new QFrame (stateFrame, "state");
	QGridLayout* infoLayout = new QGridLayout (infoFrame, 5, 4, 6, 4);
	infoLayout->setMargin (6);
	infoLayout->setColStretch (2, 1);
	infoLayout->setRowStretch (0, 1);

	label = new QLabel ("OK:", infoFrame);
	infoLayout->addWidget (label, 0, 0, Qt::AlignRight | Qt::AlignBottom);	

	_okLabel = new QLabel ("0", infoFrame, "ok");
	_okLabel->setAlignment (Qt::AlignRight);
	infoLayout->addWidget (_okLabel, 0, 1, Qt::AlignLeft | Qt::AlignBottom);

	label = new QLabel ("Failed:", infoFrame);
	infoLayout->addWidget (label, 1, 0, Qt::AlignRight);	

	_failedLabel = new QLabel ("0", infoFrame, "failed");
	_failedLabel->setAlignment (Qt::AlignRight);
	infoLayout->addWidget (_failedLabel, 1, 1, Qt::AlignLeft);

	label = new QLabel ("Skipped:", infoFrame);
	infoLayout->addWidget (label, 2, 0, Qt::AlignRight);	

	_skippedLabel = new QLabel ("0", infoFrame, "skipped");
	_skippedLabel->setAlignment (Qt::AlignRight);
	infoLayout->addWidget (_skippedLabel, 2, 1, Qt::AlignLeft);

	QFrame* lineFrame = new QFrame (infoFrame, "line");
	lineFrame->setFrameStyle (QFrame::HLine | QFrame::Plain);
	lineFrame->setMinimumHeight (6);
	infoLayout->addWidget (lineFrame, 3, 1, Qt::AlignLeft);

	_encounteredLabel = new QLabel ("0", infoFrame, "executed");
	_encounteredLabel->setAlignment (Qt::AlignRight);
	infoLayout->addWidget (_encounteredLabel, 4, 1, Qt::AlignLeft);

	label = new QLabel ("Failures:", infoFrame);
	infoLayout->addWidget (label, 0, 2, Qt::AlignRight | Qt::AlignBottom);	

	_failuresLabel = new QLabel ("0", infoFrame, "failures");
	_failuresLabel->setAlignment (Qt::AlignRight);
	infoLayout->addWidget (_failuresLabel, 0, 3, Qt::AlignLeft | Qt::AlignBottom);	

	label = new QLabel ("Errors:", infoFrame);
	infoLayout->addWidget (label, 1, 2, Qt::AlignRight);	

	_errorsLabel = new QLabel ("0", infoFrame, "errors");
	_errorsLabel->setAlignment (Qt::AlignRight);
	infoLayout->addWidget (_errorsLabel, 1, 3, Qt::AlignLeft);	

	int w = (_encounteredLabel->fontMetrics ()).width ("0000");
	      _okLabel->setMinimumWidth (w);
	  _failedLabel->setMinimumWidth (w);
	 _skippedLabel->setMinimumWidth (w);
	_failuresLabel->setMinimumWidth (w);
	  _errorsLabel->setMinimumWidth (w);
	    _encounteredLabel->setMinimumWidth (w);
	     lineFrame->setMinimumWidth (w);
	infoLayout->addColSpacing (2, w * 4);
	
	paneLayout->addWidget (infoFrame, 1);

	stateLayout->addLayout (paneLayout, 1);

	stateFrame->adjustSize ();
	layout->addWidget (stateFrame, 0, 0);

	QFrame* mainFrame = new QFrame (this, "terminal");
	mainFrame->setFrameStyle (QFrame::Panel | QFrame::Raised);
	QGridLayout* mainLayout = new QGridLayout (mainFrame, 2, 3, 6, 4);
	mainLayout->setColStretch (0, 1);
	mainLayout->setRowStretch (1, 1);

	_suiteSelector = new QComboBox (false, mainFrame, "suite");
	mainLayout->addWidget (_suiteSelector, 0, 0);

	_controlButton = new QPushButton (mainFrame, "control");
	_controlButton->setText ("Start");
	mainLayout->addWidget (_controlButton, 0, 1);	

	_quitButton = new QPushButton (mainFrame, "quit");
	_quitButton->setText ("Quit");
	mainLayout->addWidget (_quitButton, 0, 2);	

	_logView = new QListView (mainFrame, "log");
	_logView->addColumn ("#"); // No. Test
	_logView->addColumn ("!"); // Error or Failure
	_logView->addColumn ("Origin");
	_logView->addColumn ("Phase");
	_logView->addColumn ("Exception");
	_logView->setAllColumnsShowFocus (true);
	_logView->setSorting (0, true);
	_logView->setMinimumSize (QSize (400, 320));
	mainLayout->addMultiCellWidget (_logView, 1, 1, 0, 2);

	mainFrame->adjustSize ();
	layout->addWidget (mainFrame, 1, 0);

	connect (_controlButton, SIGNAL(clicked()), this, SLOT(controlTests()));
	connect (_suiteSelector, SIGNAL(activated(int)), this, SLOT(setSuite(int)));
	connect (_quitButton,  SIGNAL(clicked()), qApp, SLOT(quit()));

	_actSuite	= 0;
	_testing	= false;

	QPixmap icon (const_cast<const char**>(Scrunitizer_xpm));
	setIcon (icon);

	ExRecItem::acquire ();
}


QScrunitizer::~QScrunitizer () {
	ExRecItem::release ();
}


void QScrunitizer::reset () {
	if (_manager != 0) _manager->reset ();

	//QColor c (0, 0, 200);
	QColor c (220, 220, 220);
	_progressIndicator->setCylinderColor (c);
	_progressIndicator->reset ();
	_logView->clear ();
	_logView->clearSelection ();
 	         _okLabel->setText ("0");
 	     _failedLabel->setText ("0");
 	    _skippedLabel->setText ("0");
	   _failuresLabel->setText ("0");	
	     _errorsLabel->setText ("0");
 	_encounteredLabel->setText ("0");

	qApp->processEvents (5);
}


bool QScrunitizer::configureFrom (Scrunitizer::ArgVector& args) {
	return _manager != 0;
}


void QScrunitizer::beginSession (Scrunitizer::TestSession& session) {
	// Nothing to do
}


void QScrunitizer::endSession (Scrunitizer::TestSession& session, bool cancelled) {
	if (session.noErrors () == 0 && session.noFailures () == 0 && session.noSkipped () == 0) {
		QColor c (30, 240, 0);
		_progressIndicator->setCylinderColor (c);
	}
}


void QScrunitizer::beginTest  (Scrunitizer::Test& test, Scrunitizer::TestSession& session) {
	// Nothing to do
}


void QScrunitizer::endTest (Scrunitizer::Test& test, Scrunitizer::TestSession& session, bool success) {
	if (test.isTestCase ()) {
		_progressIndicator->setProgress (session.noEncountered ());
	}

	QString no;

	no.setNum (session.noPassed ());
	_okLabel->setText (no);

	no.setNum (session.noFailed ());
	_failedLabel->setText (no);

	no.setNum (session.noSkipped ());	
	_skippedLabel->setText (no);

	no.setNum (session.noEncountered ());
	_encounteredLabel->setText (no);

	qApp->processEvents (5);
}


void QScrunitizer::logException (Scrunitizer::TestExceptionRecord& er, Scrunitizer::TestSession& session) {
	QString msg;
	QString no;

	new ExRecItem (_logView, er, session.noErrors () + session.noFailures ());
	if (er.isErrorRecord ()) {
		no.setNum (session.noErrors ());
		_errorsLabel->setText (no);

		if (session.noErrors () == 1) {
			QColor c (200, 0, 0);
			_progressIndicator->setCylinderColor (c);
		}
	} else {
		no.setNum (session.noFailures ());	
		_failuresLabel->setText (no);

		if (session.noErrors () == 0 && session.noFailures () == 1) {
			// QColor c (150, 100, 40);
			QColor c (180, 80, 30);
			_progressIndicator->setCylinderColor (c);
		}
	}
}


void QScrunitizer::testSkipped (Scrunitizer::Test& test, Scrunitizer::TestSession& session) {
	_progressIndicator->setProgress (session.noEncountered ());

	QString no;

	no.setNum (session.noSkipped ());	
	_skippedLabel->setText (no);

	no.setNum (session.noEncountered ());
	_encounteredLabel->setText (no);
}



void QScrunitizer::scrutinize (Scrunitizer::TestManager& manager, Scrunitizer::Test& test) {
	vector<Scrunitizer::Test*> tests;
	tests.push_back (&test);

	scrutinize (manager, tests);
}


void QScrunitizer::scrutinize (Scrunitizer::TestManager& manager, vector<Scrunitizer::Test*>& tests) {
	_manager	= &manager;
	_tests		= new vector<Scrunitizer::Test*> (tests);
	_actSuite	= 0;

	vector<Scrunitizer::Test*>::iterator pivot;
	for (pivot = _tests->begin (); pivot != _tests->end (); ++pivot) {
		_suiteSelector->insertItem (((*pivot)->getName ()).c_str ());
	}
	_progressIndicator->setTotalSteps (0);
	_progressIndicator->setProgress (0);
}


void QScrunitizer::controlTests () {
	if (_testing) {
		_manager->tryCancel ();
	} else {
		reset ();

		_noTestCases	= ((*_tests)[_actSuite])->noTestCases ();
		_progressIndicator->setTotalSteps (_noTestCases);

		_quitButton->setEnabled (false);
		_suiteSelector->setEnabled (false);
		_controlButton->setText ("Stop");

		_testing = true;
		_manager->conduct (*((*_tests)[_actSuite]));
		_testing = false;

		_controlButton->setText ("Start");
		_suiteSelector->setEnabled (true);
		_quitButton->setEnabled (true);
	}
}


void QScrunitizer::setSuite (int no) {
	_actSuite = no;
}
