/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.1 / Last changed: 22.07.00
*/


#include <math.h>

#include <qpainter.h>
#include <qcolor.h>

#include <QXProgressCylinder.hh>


static inline double dmap (double v, double lowerBound, double upperBound) {
	if (v < lowerBound) return lowerBound;
	if (v > upperBound) return upperBound;
	return v;
}


QXProgressCylinder::QXProgressCylinder (QWidget* parent, const char* name, WFlags f) 
 : QProgressBar (parent, name, f), 
  _cylinderColor (0, 0, 200),
  _lightAngle (45.0),
  _cylinderDarkening (0.5),
  _grooveDarkening (0.7),
  _cylinderReflectPower (12),
  _grooveReflectPower (8),
  _grooveRelighting (0.5) {
	setFrameStyle (QFrame::Box | QFrame::Plain);
	setLineWidth (0);
}


QXProgressCylinder::QXProgressCylinder (int totalSteps, QWidget* parent, const char* name, WFlags f)
 : QProgressBar (totalSteps, parent, name, f), 
  _cylinderColor (0, 0, 200),
  _lightAngle (45.0),
  _cylinderDarkening (0.5),
  _grooveDarkening (0.7),
  _cylinderReflectPower (12),
  _grooveReflectPower (8),
  _grooveRelighting (0.5) {
	setFrameStyle (QFrame::Box);
	setLineWidth (0);
}


QColor& QXProgressCylinder::cylinderColor () {
	return _cylinderColor;
}


void QXProgressCylinder::setCylinderColor (QColor& color) {
	_cylinderColor = color;
	QPainter p (this);
	drawContents (&p);
}


void QXProgressCylinder::getLighting (double& lightAngle, 
				      double& cylinderDarkening, double& grooveDarkening,
				      int& cylinderReflectPower, int& grooveReflectPower,
				      double& grooveRelighting) {
	lightAngle		= _lightAngle;
	cylinderDarkening	= _cylinderDarkening;
	grooveDarkening		= _grooveDarkening;
	cylinderReflectPower	= _cylinderReflectPower;
	grooveReflectPower	= _grooveReflectPower;
	grooveRelighting	= _grooveRelighting;
}


void QXProgressCylinder::setLighting (double lightAngle, 
				      double cylinderDarkening, double grooveDarkening,
				      int cylinderReflectPower, int grooveReflectPower,
				      double grooveRelighting) {
	_lightAngle		= dmap (lightAngle, 0.0, 180.0);
	_cylinderDarkening	= dmap (cylinderDarkening, 0.0, 1.0);
	_grooveDarkening	= dmap (grooveDarkening, 0.0, 1.0);
	_cylinderReflectPower	= (cylinderReflectPower < 1) ? 1 : cylinderReflectPower;
	_grooveReflectPower	= (grooveReflectPower < 1) ? 1 : grooveReflectPower;
	_grooveRelighting	= dmap (grooveRelighting, 0.0, 1.0);
}


static const double pi = 3.1415927;
static const double pi2 = 3.1415927 / 2.0;

void QXProgressCylinder::drawContents (QPainter* p) {
	// First, get the drawable area
	const QRect	bar = contentsRect();

	// Let's see how much space is needed for the text on the right
	QFontMetrics	fm = p->fontMetrics ();
	int		rightGap = fm.width (QString::fromLatin1 ("000%"));

	// On the left, space is needed for the outstanding part of the cylinder
	int leftGap = (int)((bar.height () - 2.0) / 2.0 * 0.6 + 0.5);

	// Total width
	int cylinderWidth = bar.width () - leftGap - rightGap - 4;

	double cylinderRad = (bar.height () - 2.0) / 2.0;
	double alpha;
	double normAlpha;

	int cylinderHue;
	int cylinderSat;
	int cylinderVal;
	_cylinderColor.getHsv (cylinderHue, cylinderSat, cylinderVal);
	cylinderVal = (int)(cylinderVal * _cylinderDarkening);
	QColor cylinderCol;
	QColor cylinderEndCol;
	cylinderEndCol.setHsv (cylinderHue, cylinderSat, cylinderVal);

	int grooveHue;
	int grooveSat;
	int grooveVal;
	((colorGroup ()).background ()).getHsv (grooveHue, grooveSat, grooveVal);
	grooveVal = (int)(grooveVal * _grooveDarkening);
	QColor grooveCol;
	QColor grooveBegCol;
	grooveBegCol.setHsv (grooveHue, grooveSat, grooveVal);

	int mx = (totalSteps () > 0 && progress () > 0) ? (cylinderWidth * progress () / totalSteps ()) : 0;
	int bx = bar.x () + leftGap;
	int by;
	int dx;
	int ex;

	p->setPen ((colorGroup ()).shadow ());
	by = bar.y ();
	ex = bx + cylinderWidth;
	p->drawLine (bx - 1, by, ex, by);
	p->drawLine (bx - 1, by, bx - 1, by + bar.height () - 2);
	p->setPen ((colorGroup ()).light ());
	by = bar.y () + bar.height () - 1;
	ex = ex + 1;
	p->drawLine (bx - 1, by, ex, by);
	p->drawLine (ex, bar.y (), ex, by);

	int i;
	for (i = 0; i < bar.height () - 2; i++) {
		by	= bar.y () + 1 + i;
		alpha	= asin (- (cylinderRad - i) / cylinderRad);
		dx	= (int)(cos (alpha) * (bar.height () - 2.0) / 2.0 * 0.6 + 0.5);
		if (dx == 0) dx = 1;

		if (mx == 0) {
			p->setPen (grooveBegCol);
			p->drawLine (bx, by, bx + dx, by);

			normAlpha = alpha - (_lightAngle * pi / 180.0);
			normAlpha = (normAlpha < -pi2) ? -pi2 : ((normAlpha > pi2) ? pi2 : normAlpha); 
			grooveCol.setHsv (
			  grooveHue,
			  grooveSat - (int)(grooveSat * pow (cos (normAlpha), _grooveReflectPower)),
			  grooveVal + (int)((255 - grooveVal) * cos (normAlpha) * _grooveRelighting) );
			p->setPen (grooveCol);
			p->drawLine (bx + dx + 1, by, bx + cylinderWidth, by);
		} else {
			normAlpha = alpha + (_lightAngle * pi / 180.0);
			normAlpha = (normAlpha < -pi2) ? -pi2 : ((normAlpha > pi2) ? pi2 : normAlpha); 
			cylinderCol.setHsv (
			  cylinderHue,
			  cylinderSat - (int)(cylinderSat * pow (cos (normAlpha), _cylinderReflectPower)),
			  cylinderVal + (int)((255 - cylinderVal) * cos (normAlpha)) );
			p->setPen (cylinderCol);
			p->drawLine (bx - dx + 1, by, bx - dx + mx, by);

			ex = bx + dx + mx;
			p->setPen (cylinderEndCol);
			if (ex >= bx + cylinderWidth) {
				p->drawLine (bx - dx + mx + 1, by, bx + cylinderWidth , by);
			} else {
				p->drawLine (bx - dx + mx + 1, by, ex , by);

				normAlpha = alpha - (_lightAngle * pi / 180.0);
				normAlpha = (normAlpha < -pi2) ? -pi2 : 
						((normAlpha > pi2) ? pi2 : normAlpha); 
				grooveCol.setHsv (
				  grooveHue,
				  grooveSat - (int)(grooveSat * pow (cos(normAlpha), _grooveReflectPower)),
				  grooveVal + (int)((255 -grooveVal) * cos(normAlpha) * _grooveRelighting));
				p->setPen (grooveCol);
				p->drawLine (ex + 1, by, bx + cylinderWidth, by);
			}
		}
	}

	QRect textArea (bar.width () - rightGap + 4, bar.y (), rightGap - 4, bar.height ());
	p->fillRect (textArea, (colorGroup ()).brush (QColorGroup::Background));
	if (progress () >= 0 && totalSteps () > 0) {
		QString valueStr;
		valueStr.setNum (progress () * 100 / totalSteps ());
		valueStr += "%";
		p->setPen ((colorGroup ()).text ());
		p->drawText(bar, AlignRight | AlignVCenter, valueStr);
	}
}


void QXProgressCylinder::drawContentsMask (QPainter*) {
}


bool QXProgressCylinder::setIndicator (QString& progress_str, int progress, int totalSteps) {
	return QProgressBar::setIndicator (progress_str, progress, totalSteps);
}


void QXProgressCylinder::styleChange (QStyle&) {
}



// **********************************************************************************************
