/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.4 / Last changed: 29.10.00
*/


#include <string>
#include <strstream>

#include <sys/time.h>
#include <unistd.h>

#include <Callable.hh>
#include <TestCase.hh>
#include <TestCaseAdapter.hh>
#include <TestSuite.hh>
#include <TestSuiteAdapter.hh>
#include <TestExceptions.hh>
#include <TestGadget.hh>
#include <TestAppliance.hh>
#include <TestRepeater.hh>
#include <TestProxy.hh>
#include <TestSession.hh>
#include <TestManager.hh>
#include <Assertions.hh>
#include <ExceptionTrap.hh>
#include <ExceptionHandler.hh>

#include <DemoTest.hh>


//***************************************************************
//                Test State

static Scrunitizer::TestSession*	session			= 0;
static int				expectedFailures	= 0;
static int				expectedErrors		= 0;
static int				expectedSkipped		= 0;
static int				opDelay			= 100;


void resetCounters () {
	expectedFailures	= 0;
	expectedErrors		= 0;
	expectedSkipped		= 0;
}


void resetSession (Scrunitizer::TestSession* s) {
	session = s;
	resetCounters ();
}

int getExpectedFailures () {
	return expectedFailures;
}


int getExpectedErrors () {
	return expectedErrors;
}


int getExpectedSkipped () {
	return expectedSkipped;
}


void enableDelay () {
	opDelay = 100;
}


void disableDelay () {
	opDelay = 0;
}


void opPause (int msec) {
	if (msec <= 0) return;

	struct timeval delay;

	delay.tv_sec	=  msec / 1000;
	delay.tv_usec	= (msec % 1000) * 1000;

	fd_set dummy;
	FD_ZERO (&dummy);

	select (0, &dummy, &dummy, &dummy, &delay);
}


void opPause () {
	opPause (opDelay);
}


//***************************************************************
//                Test Exceptions

struct TagException {
};


struct TagFailure : public TagException {
};


struct TagError : public TagException {
};

//***************************************************************
//                Test Functions

void noOp () {
	opPause ();
}


void throwFailure () {
	opPause ();
	++expectedFailures;
	throw TagFailure ();
}


void throwError () {
	opPause ();
	++expectedErrors;
	throw TagError ();
}

void throwSupplementaryFailure () {
	opPause ();
	++expectedFailures;
	throw TagFailure ();
}

void throwSupplementaryError () {
	opPause ();
	++expectedErrors;
	throw TagError ();
}


void crashTest () {
	opPause ();
	int* ptr = 0;
	*ptr = 0;
}


void expectSkippedTests (int n) {
	expectedSkipped += n;
}


void binder1TestFunc (int p1) {
	opPause ();
	if (p1 == 101) {
		++expectedFailures;
		throw TagFailure ();
	}
	if (p1 == 666) {
		++expectedErrors;
		throw TagError ();
	}
}


void binder2TestFunc (int p1, int p2) {
	opPause ();
	if (p1 + p2 == 101) {
		++expectedFailures;
		throw TagFailure ();
	}
	if (p1 + p2 == 666) {
		++expectedErrors;
		throw TagError ();
	}
}


void binder3TestFunc (int p1, int p2, int p3) {
	opPause ();
	if (p1 + p2 + p3 == 101) {
		++expectedFailures;
		throw TagFailure ();
	}
	if (p1 + p2 + p3 == 666) {
		++expectedErrors;
		throw TagError ();
	}
}


void binder4TestFunc (int p1, int p2, int p3, int p4) {
	opPause ();
	if (p1 + p2 + p3 + p4 == 101) {
		++expectedFailures;
		throw TagFailure ();
	}
	if (p1 + p2 + p3 + p4 == 666) {
		++expectedErrors;
		throw TagError ();
	}
}

//***************************************************************
//                Test Fixture Class

class TestObj {

	private:
		int	_state;

	public:
		TestObj () : _state (0) {}

		void mfNoOp ();
		void mfThrowFailure ();
		void mfThrowError ();
		void mfFulfilledCondAssertion ();
		void mfFulfilledEqualAssertion ();
		void mfFulfilledDeltaEqualAssertion ();
		void mfFailedCondAssertion ();
		void mfFailedDeltaEqualAssertion ();
		void mfFailedEqualAssertion ();

		void apply1TestFunc (int p1);
		void apply2TestFunc (int p1, int p2);
		void apply3TestFunc (int p1, int p2, int p3);
		void apply4TestFunc (int p1, int p2, int p3, int p4);
};


void TestObj::mfNoOp () {
	opPause ();
}

	
void TestObj::mfThrowFailure () {
	opPause ();
	++expectedFailures;
	throw TagFailure ();
}


void TestObj::mfThrowError () {
	opPause ();
	++expectedErrors;
	throw TagError ();
}


void TestObj::mfFulfilledCondAssertion () {
	opPause ();
	assertCondition (_state == 0, "Expected fulfilled assertion");
}


void TestObj::mfFulfilledEqualAssertion () {
	opPause ();
	assertEqual (_state, 0, "Expected fulfilled equal assertion");
}


void TestObj::mfFulfilledDeltaEqualAssertion () {
	opPause ();
	double v = _state + 0.1;
	assertApprox (v, 0.2, 0.15, "Expected fulfilled epsilon equal assertion");
}


void TestObj::mfFailedCondAssertion () {
	opPause ();
	++expectedFailures;
	assertCondition (_state < 0, "Expected failed assertion");
}


void TestObj::mfFailedEqualAssertion () {
	opPause ();
	++expectedFailures;
	assertEqual (_state, 1, "Expected failed equal assertion");
}


void TestObj::mfFailedDeltaEqualAssertion () {
	opPause ();
	double v = _state + 0.1;
	++expectedFailures;
	assertApprox (v, 1.0, 0.1, "Expected failed epsilon equal assertion");
}


void TestObj::apply1TestFunc (int p1) {
	opPause ();
	if (p1 == 101) {
		++expectedFailures;
		throw TagFailure ();
	}
	if (p1 == 666) {
		++expectedErrors;
		throw TagError ();
	}
}


void TestObj::apply2TestFunc (int p1, int p2) {
	opPause ();
	if (p1 + p2 == 101) {
		++expectedFailures;
		throw TagFailure ();
	}
	if (p1 + p2 == 666) {
		++expectedErrors;
		throw TagError ();
	}
}


void TestObj::apply3TestFunc (int p1, int p2, int p3) {
	opPause ();
	if (p1 + p2 + p3 == 101) {
		++expectedFailures;
		throw TagFailure ();
	}
	if (p1 + p2 + p3 == 666) {
		++expectedErrors;
		throw TagError ();
	}
}


void TestObj::apply4TestFunc (int p1, int p2, int p3, int p4) {
	opPause ();
	if (p1 + p2 + p3 + p4 == 101) {
		++expectedFailures;
		throw TagFailure ();
	}
	if (p1 + p2 + p3 + p4 == 666) {
		++expectedErrors;
		throw TagError ();
	}
}


//***************************************************************
//                Exception Handlers

void handleTagFailure (TagFailure& ex) {
	throw Scrunitizer::TestFailure ("Tag failure");
}

void handleTagError (TagError& ex) {
	throw Scrunitizer::TestError ("Tag error");
}

//***************************************************************
//                    Test Set-Up

Scrunitizer::TestSuite* setUpTestSuite (
	const string&	name,
	bool		failures,
	bool		errors,
	bool		delayed,
	int		iterations
) {
	using Scrunitizer::bind;
	using Scrunitizer::apply;
	using Scrunitizer::makeTestCase;
	using Scrunitizer::makeTestSuite;
	using Scrunitizer::makeTestGadget;

	Scrunitizer::TestSuite* suite;

	if (delayed) {
		suite = makeTestSuite (name, false, enableDelay);
	} else {
		suite = makeTestSuite (name, false, disableDelay);
	}

	suite->add (makeTestCase ("Auto-generated NOP test",  noOp));
	suite->add (makeTestCase ("Created NOP test", noOp));
	suite->add (makeTestCase ("Full featured NOP test", noOp, noOp, noOp));

	suite->add (makeTestCase ("Binder1 test", bind (binder1TestFunc, 17)));
	suite->add (makeTestCase ("Binder2 test", bind (binder2TestFunc, 10, 7)));
	suite->add (makeTestCase ("Binder3 test", bind (binder3TestFunc, 8, 8, 1)));
	suite->add (makeTestCase ("Binder4 test", bind (binder4TestFunc, 8, 5, 3, 1)));

	if (failures) {
		suite->add (makeTestCase ("Expected failure in test",
					  throwFailure));
		suite->add (makeTestCase ("Expected failure in set-up", 
					  noOp,  throwFailure));
		suite->add (makeTestCase ("Expected failure in tear-down", 
					  noOp, noOp,  throwFailure));

		suite->add (makeTestCase ("Expected failure via binder1", 
					  bind (binder1TestFunc, 101)));
		suite->add (makeTestCase ("Expected failure via binder2", 
					  bind (binder2TestFunc, 50, 51)));
		suite->add (makeTestCase ("Expected failure via binder3", 
					  bind (binder3TestFunc, 50, 30, 21)));
		suite->add (makeTestCase ("Expected failure via binder4", 
					  bind (binder4TestFunc, 40, 30, 20, 11)));
	}

	// suite->add (makeTestCase ("Crash test", crashTest));

	if (errors) {
		suite->add (makeTestCase ("Expected error in test",  throwError));
		suite->add (makeTestCase ("Expected error in set-up", noOp, throwError));
		suite->add (makeTestCase ("Expected error in tear-down", noOp, noOp,  throwError));

		if (failures) {
			suite->add (makeTestCase (
					"Expected premature tear-down due to failure after failure",
						  throwFailure,  noOp,  throwSupplementaryFailure));
			suite->add (makeTestCase ("Expected premature tear-down due to error after failure",
				     		  throwFailure,  noOp,  throwSupplementaryError));
			suite->add (makeTestCase ("Expected premature tear-down due to failure after error",
						  throwError,  noOp,  throwSupplementaryFailure));
			suite->add (makeTestCase ("Expected premature tear-down due to error after error",
				     		  throwError,  noOp,  throwSupplementaryError));
		}

		Scrunitizer::TestSuite* suesuite = makeTestSuite ("Setup error suite", false, noOp);
		suesuite->add (new Scrunitizer::TestRepeater ("Take 5", 5,
				makeTestCase ("Created NOP test", noOp)));
		suesuite->add (makeTestCase ("Full featured NOP test", noOp, noOp, noOp));

		suite->add (makeTestCase ("Expected error via binder1", 
					  bind (binder1TestFunc, 666)));
		suite->add (makeTestCase ("Expected error via binder2", 
					  bind (binder2TestFunc, 333, 333)));
		suite->add (makeTestCase ("Expected error via binder3",
					  bind (binder3TestFunc, 222, 222, 222)));
		suite->add (makeTestCase ("Expected error via binder4",
					  bind (binder4TestFunc, 222, 222, 111, 111)));

		suite->add (suesuite);

	}

	Scrunitizer::TestSuite* osuite = new Scrunitizer::TestSuite ("Object test suite");
	osuite->add (makeTestGadget (	"Member function NOP test", 
					apply (&TestObj::mfNoOp)));
	osuite->add (makeTestGadget (	"Member function condition assertion", 
					apply (&TestObj::mfFulfilledCondAssertion)));
	osuite->add (makeTestGadget (	"Member function euqal assertion", 
					apply (&TestObj::mfFulfilledEqualAssertion)));
	osuite->add (makeTestGadget (	"Member function delta equal assertion", 
					apply (&TestObj::mfFulfilledDeltaEqualAssertion)));

	osuite->add (makeTestGadget ("Apply1 test", apply (&TestObj::apply1TestFunc, 17)));
	osuite->add (makeTestGadget ("Apply2 test", apply (&TestObj::apply2TestFunc, 10, 7)));
	osuite->add (makeTestGadget ("Apply3 test", apply (&TestObj::apply3TestFunc, 8, 8, 1)));
	osuite->add (makeTestGadget ("Apply4 test", apply (&TestObj::apply4TestFunc, 8, 5, 3, 1)));

	if (failures) {

		osuite->add (makeTestGadget (	"Expected member function failure", 
						apply (&TestObj::mfThrowFailure)));
		osuite->add (makeTestGadget (	"Expected member function condition assertion failure",
						 apply (&TestObj::mfFailedCondAssertion)));
		osuite->add (makeTestGadget (	"Expected member function equal assertion failure", 
						apply (&TestObj::mfFailedEqualAssertion)));
		osuite->add (makeTestGadget (	"Expected member function delta equal assertion failure", 
						apply (&TestObj::mfFailedDeltaEqualAssertion)));

		osuite->add (makeTestGadget (	"Expected failure via apply1", 
						apply (&TestObj::apply1TestFunc, 101)));
		osuite->add (makeTestGadget (	"Expected failure via apply2", 
						apply (&TestObj::apply2TestFunc, 50, 51)));
		osuite->add (makeTestGadget (	"Expected failure via apply3", 
						apply (&TestObj::apply3TestFunc, 50, 30, 21)));
		osuite->add (makeTestGadget (	"Expected failure via apply4", 
						apply (&TestObj::apply4TestFunc, 40, 30, 20, 11)));
	}
	if (errors) {
		osuite->add (makeTestGadget (	"Expected member function error", 
						apply (&TestObj::mfThrowError)));

		osuite->add (makeTestGadget (	"Expected error via apply1", 
						apply (&TestObj::apply1TestFunc, 666)));
		osuite->add (makeTestGadget (	"Expected error via apply2", 
						apply (&TestObj::apply2TestFunc, 333, 333)));
		osuite->add (makeTestGadget (	"Expected error via apply3",
						apply (&TestObj::apply3TestFunc, 222, 222, 222)));
		osuite->add (makeTestGadget (	"Expected error via apply4",
						apply (&TestObj::apply4TestFunc, 222, 222, 111, 111)));

	}

	suite->add (new Scrunitizer::TestRepeater ("Stress suite", iterations, osuite));

	if (failures) {
		Scrunitizer::TestSuite* sk1suite = makeTestSuite ("Skipping test suite", true,
								  bind (expectSkippedTests, 4));
		sk1suite->add (makeTestCase ("Expected failure to trigger skipping tests", throwFailure));
		sk1suite->add (makeTestCase ("Created test B0", noOp));

		Scrunitizer::TestSuite* sk2suite = new Scrunitizer::TestSuite ("Embedded test suite", true);
		sk2suite->add (makeTestCase ("Created test E1", noOp));
		sk2suite->add (makeTestCase ("Created test E2", noOp));
		sk2suite->add (makeTestCase ("Created test E3", noOp));

		sk1suite->add (sk2suite);

		suite->add (sk1suite);
	}

	
	Scrunitizer::TestCase *shared = makeTestCase ("Shared test", noOp);
	suite->add (shared);
	suite->add (new Scrunitizer::TestProxy (shared));

	return suite;
}


void instrumentConductor (Scrunitizer::TestManager* conductor) {
	conductor->addCallGate (new Scrunitizer::ExceptionTrap<TagFailure> ("Tag Failure"));
	conductor->addCallGate (new Scrunitizer::ExceptionHandler<TagError> (handleTagError));
	// conductor->addCallGate (new Scrunitizer::ExceptionHandler<TagFailure> (handleTagFailure));
}

//***************************************************************
