/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 05.08.00
*/


#ifndef _APPLICABLE_H_
#define _APPLICABLE_H_


#include <TestCase.hh>


namespace Scrunitizer {


//: Interface template for an object that could be applied to another object.
// Such items are typically member function adapters,
// so call wrappers for member function pointers. The class templates
// ApplicableBinder0 to ApplicableBinder4 implement this interface.
//
template<class T> class Applicable {

	public:
		//: Default destructor.
		// No functionality. Enforces that all derived
		// classes will have a virtual destructor.
		//
		virtual ~Applicable () {}

	public:
		//: Apply the item to an object.
		// Pure virtual. Needs to be defined by a derived interface template.
		//
		virtual void operator() (T*) = 0;

};



//: Call a simple (generator) member function via the Applicable<T> interface.
// This is an adapter template for parameter-less member functions to the Applicable 
// interface. Besides this adapter for parameter-less member functions, there exist
// adapters for member functions with 1 to 4 parameters, namely ApplicableBinder1 to ApplicableBinder4.
// For each adapter to Applicable<T> exists a template function 
// that could be used to instantiate the template on the fly.
// <P>
// <TT><PRE>
//template&lt;class T, typename R&gt; inline Applicable&lt;T&gt;* apply (R (T::*meth)());
//
//template&lt;class T, typename R, typename A1&gt;
//      inline Applicable&lt;T&gt;* apply (R (T::*meth)(A1), A1 arg1);
//
//template&lt;class T, typename R, typename A1, typename A2&gt; 
//      inline Applicable&lt;T&gt;* apply (R (T::*meth)(A1,A2), A1 arg1, A2 arg2);
//
//template&lt;class T, typename R, typename A1, typename A2, typename A3&gt; 
//      inline Applicable&lt;T&gt;* apply (R (T::*meth)(A1,A2,A3), A1 arg1, A2 arg2, A3 arg3);
//
//template&lt;class T, typename R, typename A1, typename A2, typename A3, typename A4&gt; 
//      inline Applicable&lt;T&gt;* apply (R (T::*meth)(A1,A2,A3,A4), A1 arg1, A2 arg2, A3 arg3, A4 arg4);
// </PRE></TT>
//
template<class T, typename R> class ApplicableBinder0 : public Applicable<T> {

	typedef R (T::*mem_func_type) ();

	private:
		mem_func_type	_func;

	public:
		//: Create an adapter for a member function.
		// This constructor takes a member function of class T (that returns type R)
		// as parameter and constructs an Applicable<T> object that could be
		// used in a TestGadjet or in a TestAppliance instance as set-up, tear-down or test action.
		//
		ApplicableBinder0 (mem_func_type func) : _func (func) {}

		//: Copy constructor.
		// Creates a copy with the same member function.
		//
		ApplicableBinder0 (const ApplicableBinder0& tmpl) : _func (tmpl._func) {}

		//: Apply the applicable to the target object.
		// Now, the member function is applied to an instance of
		// class T, which is the parameter value.
		//
		virtual void operator() (T* instance) {
			(instance->*_func) ();
		}

};

template<class T, typename R> inline Applicable<T>* apply (R (T::*meth)()) {
	return new ApplicableBinder0<T,R> (meth);
}


template<class T, typename R, typename A1> 
class ApplicableBinder1 : public Applicable<T> {

	typedef R (T::*mem_func_type) (A1);

	private:
		mem_func_type	_func;
		A1		_arg1;

	public:
		ApplicableBinder1 (mem_func_type func, A1 arg1)
			: _func (func), _arg1 (arg1) {}
		ApplicableBinder1 (const ApplicableBinder1& tmpl)
			: _func (tmpl._func), _arg1 (tmpl._arg1) {}

		virtual void operator() (T* instance) {
			(instance->*_func) (_arg1);
		}

};

template<class T, typename R, typename A1>
inline Applicable<T>* apply (R (T::*meth)(A1), A1 arg1) {
	return new ApplicableBinder1<T,R,A1> (meth, arg1);
}


template<class T, typename R, typename A1, typename A2> 
class ApplicableBinder2 : public Applicable<T> {

	typedef R (T::*mem_func_type) (A1,A2);

	private:
		mem_func_type	_func;
		A1		_arg1;
		A2		_arg2;

	public:
		ApplicableBinder2 (mem_func_type func, A1 arg1, A2 arg2)
			: _func (func), _arg1 (arg1), _arg2 (arg2) {}
		ApplicableBinder2 (const ApplicableBinder2& tmpl)
			: _func (tmpl._func), _arg1 (tmpl._arg1), _arg2 (tmpl._arg2) {}

		virtual void operator() (T* instance) {
			(instance->*_func) (_arg1, _arg2);
		}

};

template<class T, typename R, typename A1, typename A2> 
inline Applicable<T>* apply (R (T::*meth)(A1,A2), A1 arg1, A2 arg2) {
	return new ApplicableBinder2<T,R,A1,A2> (meth, arg1, arg2);
}


template<class T, typename R, typename A1, typename A2, typename A3> 
class ApplicableBinder3 : public Applicable<T> {

	typedef R (T::*mem_func_type) (A1,A2,A3);

	private:
		mem_func_type	_func;
		A1		_arg1;
		A2		_arg2;
		A3		_arg3;

	public:
		ApplicableBinder3 (mem_func_type func, A1 arg1, A2 arg2, A3 arg3)
			: _func (func), _arg1 (arg1), _arg2 (arg2), _arg3 (arg3) {}
		ApplicableBinder3 (const ApplicableBinder3& tmpl)
			: _func (tmpl._func), _arg1 (tmpl._arg1), _arg2 (tmpl._arg2), _arg3 (tmpl._arg3) {}

		virtual void operator() (T* instance) {
			(instance->*_func) (_arg1, _arg2, _arg3);
		}

};

template<class T, typename R, typename A1, typename A2, typename A3> 
inline Applicable<T>* apply (R (T::*meth)(A1,A2,A3), A1 arg1, A2 arg2, A3 arg3) {
	return new ApplicableBinder3<T,R,A1,A2,A3> (meth, arg1, arg2, arg3);
}


template<class T, typename R, typename A1, typename A2, typename A3, typename A4> 
class ApplicableBinder4 : public Applicable<T> {

	typedef R (T::*mem_func_type) (A1,A2,A3,A4);

	private:
		mem_func_type	_func;
		A1		_arg1;
		A2		_arg2;
		A3		_arg3;
		A4		_arg4;

	public:
		ApplicableBinder4 (mem_func_type func, A1 arg1, A2 arg2, A3 arg3, A4 arg4)
			: _func (func), _arg1 (arg1), _arg2 (arg2), _arg3 (arg3), _arg4 (arg4) {}
		ApplicableBinder4 (const ApplicableBinder4& tmpl)
			: _func (tmpl._func), _arg1 (tmpl._arg1), _arg2 (tmpl._arg2), 
					      _arg3 (tmpl._arg3), _arg4 (tmpl._arg4) {}

		virtual void operator() (T* instance) {
			(instance->*_func) (_arg1, _arg2, _arg3, _arg4);
		}

};

template<class T, typename R, typename A1, typename A2, typename A3, typename A4> 
inline Applicable<T>* apply (R (T::*meth)(A1,A2,A3,A4), A1 arg1, A2 arg2, A3 arg3, A4 arg4) {
	return new ApplicableBinder4<T,R,A1,A2,A3,A4> (meth, arg1, arg2, arg3, arg4);
}


} // End of namespace Scrunitizer
#endif
