/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 05.08.00
*/


#ifndef _BINDER_H_
#define _BINDER_H_


namespace Scrunitizer {

//: Binds values to the parameters of a function.
// A binder is a special kind of adapter that calls a function via a pointer
// with a predefined - and therefore constant - value for each of its parameters.
// There exist binders for functions with 1 to 4 arguments, Binder1 to Binder4
// <P>
// A convenience function is provided for each binder (<TT>bind</TT>).
// Using this function removes the burden of manually specifying the
// function and the argument type when instantiating one of the binder template.
// Scrunitizer has binders for functions from 1 to 4 parameters (type A1- A4) that
// could return an arbitrary type (type R). The return value is always ignored.
// <P>
// <TT><PRE>
//template&lt;typename R, typename A1&gt; 
//      inline Binder1&lt;R,A1&gt; bind (R (*func)(A1), A1 arg);
//
//template&lt;typename R, typename A1, typename A2&gt; 
//      inline Binder2&lt;R,A1,A2&gt; bind (R (*func)(A1,A2), A1 arg1, A2 arg2);
//
//template&lt;typename R, typename A1, typename A2, typename A3&gt; 
//      inline Binder3&lt;R,A1,A2,A3&gt; bind (R (*func)(A1,A2,A3), A1 arg1, A2 arg2, A3 arg3);
//
//template&lt;typename R, typename A1, typename A2, typename A3, typename A4&gt; 
//      inline Binder4&lt;R,A1,A2,A3,A4&gt; bind (R (*func)(A1,A2,A3,A4), A1 arg1, A2 arg2, A3 arg3, A4 arg4);
// </PRE></TT>
//
template<typename R, typename A1> class Binder1 {

	typedef R (*func_type) (A1);

	private:
		func_type	_func;
		A1		_arg1;

	public:
		//: Create a binder (for a function with one parameter).
		// Beside the function pointer, an argument is needed for
		// the first parameter. A1 is the type of the functions 
		// single parameter.
		//
		Binder1 (func_type func, A1 arg1)
			: _func (func), _arg1 (arg1) {}

		//: Copy constructor.
		// Creates a copy of the binder (same function, same parameter value).
		//
		Binder1 (const Binder1& tmpl) 
			: _func (tmpl._func), _arg1 (tmpl._arg1) {}

		//: The function call operator.
		// It invokes the given function with the specified
		// argument value.
		//
		void operator() () {
			_func (_arg1);
		}

};

template<typename R, typename A1> 
inline  Binder1<R,A1> bind (R (*func)(A1), A1 arg) {
	return Binder1<R,A1> (func, arg);
}


template<typename R, typename A1, typename A2> class Binder2 {

	typedef R (*func_type) (A1,A2);

	private:
		func_type	_func;
		A1		_arg1;
		A2		_arg2;

	public:
		Binder2 (func_type func, A1 arg1, A2 arg2) 
			: _func (func), _arg1 (arg1), _arg2 (arg2) {}
		Binder2 (const Binder2& tmpl) 
			: _func (tmpl._func), _arg1 (tmpl._arg1), _arg2 (tmpl._arg2) {}

		void operator() () {
			_func (_arg1, _arg2);
		}

};

template<typename R, typename A1, typename A2> 
inline Binder2<R,A1,A2> bind (R (*func)(A1,A2), A1 arg1, A2 arg2) {
	return Binder2<R,A1,A2> (func, arg1, arg2);
}


template<typename R, typename A1, typename A2, typename A3> class Binder3 {

	typedef R (*func_type) (A1,A2,A3);

	private:
		func_type	_func;
		A1		_arg1;
		A2		_arg2;
		A3		_arg3;

	public:
		Binder3 (func_type func, A1 arg1, A2 arg2, A3 arg3) 
			: _func (func), _arg1 (arg1), _arg2 (arg2), _arg3 (arg3) {}
		Binder3 (const Binder3& tmpl) 
			: _func (tmpl._func), _arg1 (tmpl._arg1), _arg2 (tmpl._arg2), _arg3 (tmpl._arg3) {}

		void operator() () {
			_func (_arg1, _arg2, _arg3);
		}

};

template<typename R, typename A1, typename A2, typename A3> 
inline Binder3<R,A1,A2,A3> bind (R (*func)(A1,A2,A3), A1 arg1, A2 arg2, A3 arg3) {
	return Binder3<R,A1,A2,A3> (func, arg1, arg2, arg3);
}


template<typename R, typename A1, typename A2, typename A3, typename A4> class Binder4 {

	typedef R (*func_type) (A1,A2,A3,A4);

	private:
		func_type	_func;
		A1		_arg1;
		A2		_arg2;
		A3		_arg3;
		A4		_arg4;

	public:
		Binder4 (func_type func, A1 arg1, A2 arg2, A3 arg3, A4 arg4) 
			: _func (func), _arg1 (arg1), _arg2 (arg2), _arg3 (arg3), _arg4 (arg4) {}
		Binder4 (const Binder4& tmpl) 
			: _func (tmpl._func), _arg1 (tmpl._arg1), _arg2 (tmpl._arg2), 
                                              _arg3 (tmpl._arg3), _arg4 (tmpl._arg4) {}

		void operator() () {
			_func (_arg1, _arg2, _arg3, _arg4);
		}

};

template<typename R, typename A1, typename A2, typename A3, typename A4> 
inline Binder4<R,A1,A2,A3,A4> bind (R (*func)(A1,A2,A3,A4), A1 arg1, A2 arg2, A3 arg3, A4 arg4) {
	return Binder4<R,A1,A2,A3,A4> (func, arg1, arg2, arg3, arg4);
}


} // End of namespace Scrunitizer
#endif
