/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 30.07.00
*/


#ifndef _CALLABLE_H_
#define _CALLABLE_H_


namespace Scrunitizer {

//: Interface for an item that could be called.
// This interface offers only one generic operation - call -
// to execute an operation.
//
class Callable {

	public:
		//: Default destructor.
		// No functionality. Enforces that all derived
		// classes will have a virtual destructor.
		//
		virtual ~Callable () {}

	public:
		//: Generic invocation of an operation.
		// Call a method with no parameters that returns nothing.
		//
		virtual void call () {}
		/* ??? FIXI Due to problems with egcs 2.9.2 not pure virtual */

};


//: Something that acts as a middle-man while invoking an operation.
// The main purpose of a call gate is to change the call environment
// before and after a function is executed. In C++, this allows
// to wrap a try-catch-block around the invocation of a method.
//
class CallGate : public Callable {

	private:
		Callable*	_target;
		bool		_release;

	public:
		//: Create a new call gate.
		// If <TT>release</TT> is true, the target callable is deleted when the 
		// call gate is destroyed.
		//
		CallGate (Callable* target, bool release = true) : _target (target), _release (release) {}

		//: Default constructor.
		// Leaves the target uninitialized (NULL).
		//
		CallGate () : _target (0), _release (true) {}

		//: Destrcutor
		// If <TT>release</TT> is true, the target is deleted as well.
		//
		virtual ~CallGate () {
			if (_release) delete _target;
		}

		//: Do the call.
		// The targets call function is called.
		//
		virtual void call () {
			_target->call ();
		}

		//: Rerurn the actual call target.
		// Reader method.
		//
		virtual Callable* getTarget () const {
			return _target;
		}

		//: Set a new call target.
		// As in the constructor, the release status could be set as well.
		//
		virtual void setTarget (
			Callable* target, 
			bool release = true
		) {
			_target	= target;
			_release= release;
		}
};


} // End of namespace Scrunitizer

#endif
