/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 30.07.00
*/


#ifndef _EXCEPTION_HANDLER_H_
#define _EXCEPTION_HANDLER_H_


#include <Callable.hh>
#include <TestExceptions.hh>


namespace Scrunitizer {


//: Calls a handler function when an exception is caught.
// The exception handler template generates CallGates that catch
// an exception of type E and call a handler function as result.
// The handler function could then decide what to do (i.e. throw
// a TestFailure or TestError). An exception handler can be useful when 
// many different unit test must handle the same exception type
// while executing the test. Instead of writing try-catch-blocks
// in every unit test, an exception handler could be registered
// in the test conductor (see also TestManager).
//
template<class E> class ExceptionHandler : public CallGate {

	typedef void (*handler_func)(E&);

	private:
		handler_func	_handler;

	public:
		//: Create an exception handler instance.
		// Beside the handler function, the call target and its release
		// status could be set. If the trap is used by a TestManager, the
		// manager sets the target.
		//
		ExceptionHandler (
			handler_func handler, 
			Callable* target = 0, 
			bool release = true
		) : CallGate (target, release), _handler (handler)  {
		}

		//: Copy-constructor
		// Create another, identical exception handler.
		//
		ExceptionHandler (
			const ExceptionHandler& eh
		) : CallGate (eh), _handler (eh._handler) {
		}

	public:
		//: Calls the call target.
		// While doing that, the handler catches exceptions of type
		// E. If that happens, it calls the handler function.
		//
		virtual void call () {
			try {
				CallGate::call ();
			}
			catch (E& ex) {
				_handler (ex);
			}
		}
};


} // End of namespace Scrunitizer

#endif



