/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 30.07.00
*/


#ifndef _EXCEPTION_TRAP_H_
#define _EXCEPTION_TRAP_H_


#include <Callable.hh>
#include <TestExceptions.hh>


namespace Scrunitizer {

//: Transforms an application exception into a test failure.
// This template generates CallGates which catches an exception
// - most likely from the unit to test - and throws
// a TestFailure as result. This functionality can be useful when 
// during many different unit tests the same exception
// could be thrown. Instead of writing try-catch-blocks
// in every test, an exception trap is registered
// in the TestManager, which automatically does the job.
// 
template<class E> class ExceptionTrap : public CallGate {

	private:
		string		_message;

	public:
		//: Generate a trap that uses a fixed failure message.
		// Beside the message, a callable target and its release status could be
		// set. If the trap is used by a TestManager, the
		// conductor sets the target.
		//
		ExceptionTrap (
			const string& message, 
			Callable* target = 0, 
			bool release = true
		) : CallGate (target, release), _message (message) {
		}

		//: Copy-constructor
		// Creates an identical exception trap.
		//
		ExceptionTrap (
			const ExceptionTrap& et
		) : CallGate (*this), _message (et._message) {
		}

		//: Destructor
		// Only deletes the message string.
		//
		virtual ~ExceptionTrap () {
		};

	public:
		//: Call the target callable.
		// While doing that, exceptions of type E are caught,
		// and a TestFailure is thrown if that happens.
		//
		virtual void call () {
			try {
				CallGate::call ();
			}
			catch (E&) {
				throw TestFailure (_message);
			}
		}

};


} // End of namespace Scrunitizer

#endif



