/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 06.08.00
*/

#include <Callable.hh>

#include <ExecutionTracer.hh>


namespace Scrunitizer {


class PhaseTracker : public CallGate {

	private:
		ExecutionTracer&	_tracer;

	protected:
		PhaseTracker (
			ExecutionTracer& tracer
		) : _tracer (tracer) { }

		virtual void call () {
			_tracer.nextPhase (_tracer._manager.getCurrentTestPhase ());
			CallGate::call ();
		}

	friend class ExecutionTracer;
};


static const char* phaseName[] = {
	"  UNKNOWN",
	"  SETUP",
	"  EXECUTION",
	"  PREMATURE TEARDOWN",
	"  TEARDOWN"
};



ExecutionTracer::ExecutionTracer ( 
	TestManager&	manager,
	ostream&	stream,
	bool		enabled,
	bool		permitCancel
) : StreamingObserver (stream, enabled, permitCancel), _manager (manager), _tracker (0) {
	_tracker = new PhaseTracker (*this);
	(options ()).add (option ("-t", false, &ExecutionTracer::enabled, true))
		    .add (new LogFileOption (*this, "-tf", true, true));
}


ExecutionTracer::~ExecutionTracer () {
	delete _tracker;
}


void ExecutionTracer::beginSession (TestSession& session) {
	_manager.addCallGate (_tracker);
	log ()	<< "SESSION STARTED" << endl 
		<< flush;
}


void ExecutionTracer::endSession (TestSession& session, bool cancelled) {
	log ()	<< (cancelled ? "SESSION CANCELLED" : "SESSION COMPLETED") << endl;
	log ()	<< endl
		<< "FAILURES: " << session.noFailures () << endl
		<< "ERRORS..: " << session.noErrors () << endl
		<< "PASSED..: " << session.noPassed () << endl
		<< "FAILED..: " << session.noFailed () << endl
		<< "EXECUTED: " << session.noExecuted () << endl
		<< "SKIPPED.: " << session.noSkipped () << endl
		<< "TOTAL...: " << session.noEncountered () << endl
		<< endl
		<< flush;
	_manager.removeCallGate (_tracker);
}


void ExecutionTracer::beginTest (Test& test, TestSession& session) {
	log ()	<< (test.isTestCase () ? "TEST CASE  " : "TEST SUITE ") << test.getName () << endl
		<< flush;
}


void ExecutionTracer::endTest (Test& test, TestSession& session, bool success) {
	log ()	<< (success ? "PASSED " : "FAILED") << endl
		<< flush;
}


void ExecutionTracer::testSkipped (Test& test, TestSession& session) {
	log ()	<< "SKIP " << (test.isTestCase () ? "CASE  " : "SUITE ") << test.getName ();
	if (test.isTestCase ()) {
		log ()	<< endl << flush;
	} else {
		log ()	<< " (" << test.noTestCases () << ")" << endl << flush;
	}

}


void ExecutionTracer::logException (TestExceptionRecord& exrec, TestSession& session) {
	log ()	<< "    " << (exrec.isErrorRecord () ? "ERROR: " : "FAILURE: ")
		<< (exrec.getException ())->getMessage () << endl
		<< flush;
}


void  ExecutionTracer::nextPhase (TestPhase phase) {
	log ()	<< phaseName[phase] << endl 
		<< flush;
}


} // End of namespace Scrunitizer
