/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 03.08.00
*/

#ifndef _EXECUTION_TRACER_H_
#define _EXECUTION_TRACER_H_


#include <iostream>

#include <TestPhase.hh>
#include <TestManager.hh>
#include <StreamingObserver.hh>


namespace Scrunitizer {

class PhaseTracker;

//: Execution Trace Log.
// An execution trace log is useful in situations where a test fails
// so that the whole test driver executable crashes (i.e. because of a segmentation fault).
// In such cases, having only the a terminal log might not be enough. The 
// execution tracer logs every single step during test execution and immediately 
// flushes the output stream buffer. Because this leads to high I/O load, the 
// execution trace log should only be enabled when necessary.
// <BR>
// The output of the execution trace log is designed so that it could
// be read by humans and easily parsed by other programs.
// <P>
// See also <A HREF="observerdemo.html#trace">observer output examples</A>
//
// 
class ExecutionTracer : public StreamingObserver {

	private:
		TestManager&	_manager;
		PhaseTracker*	_tracker;

	public:
		//: Create an execution tracer.
		// Note that the default enabled-status is false, so it must be enabled
		// explicitly (typically by a command line option). 
		// <P>
		// The default option handlers for an execution tracer:<BR>
		// The flag "-t" enables the execution tracer, the flag "-tf" (trace file) enables the 
		// tracer and sets the output file. Both flags are consumed.
		//
		ExecutionTracer (
			TestManager&	manager,
			ostream&	stream,
			bool		enabled = false,
			bool		permitCancel = true
		);

		//: The destructor.
		// Does nothing special.
		//
		virtual ~ExecutionTracer ();

	public:
		//: Logs the begin of a test session.
		// From now on the execution tracer logs every single step 
		// during execution of the test session.
		//
		virtual void beginSession (TestSession& session);

		//: Logs the end of a session.
		// In addition, it is logged whether the session was cancelled or 
		// completed and a test session summary is written.
		//
		virtual void endSession   (TestSession& session, bool cancelled);

		//: A new test was started.
		// Both test cases and test suites are logged with their name.
		// At the beginning of each test phase (set-up, execution or (premature) tear-down),
		// a test phase marker is written to the log stream. This allows to exactly
		// detect in which phase a test crashes.
		//
		virtual void beginTest (Test& test, TestSession& session);

		//: A test was finished.
		// The test outcome is logged (passed or failed).
		//
		virtual void endTest   (Test& test, TestSession& session, bool success);

		//: A test was skipped.
		// Also skipping of tests is recorded to make the execution trace complete.
		// if more than one are skipped altogether, the number of skipped tests
		// is logged as well.
		//
		virtual void testSkipped (Test& test, TestSession& session);

		//: An exception is logged.
		// All kinds of exceptions are also put to the trace log.
		//
		virtual void logException (TestExceptionRecord& mr, TestSession& session);

	private:
		void nextPhase (TestPhase phase);

	//: Test phase tracker.
	// Used to track the execution phases of a single test.
	// Used only inside ExecutionTracer.
	//
	friend class PhaseTracker;
};


} // End of namespace Scrunitizer

#endif
