/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.4 / Last changed: 29.10.00
*/

#ifndef _OPTIONS_H_
#define _OPTIONS_H_


#include <vector>
#include <list>
#include <string>
#include <memory>
#include <cstring>


namespace Scrunitizer {


typedef vector<const char*> ArgVector;


//: Abstract template class for option handlers.
// An option handler is an object that interprets arguments of a command line
// to configure another instance (change its state).
// The option handler is called when its identifier appears in the argument list.
// It could consume the option identifier  - so that it is not longer
// part of the argument list - and could consumes further elements of the
// argument list that serve as configuration parameters.<BR>
// This class template servers as base for option handlers that configure 
// instances of class C.
//
template<typename C> 
class OptionHandler {

	private:
		string		_id;
		bool		_consuming;

	public:
		//: Create a option handler instance.
		// Required arguments are the option identifier and the
		// consumption status.
		//
		OptionHandler (const string& id, bool consuming) 
			: _id (id), _consuming (consuming) {}

		//: Destructor.
		// Does nothing. Ensures that all derived classes have a virtual
		// destructor.
		//
		virtual ~OptionHandler () { }

	public:
		//: The option identifier.
		// Returns the string that identifies the option.
		//
		virtual const string& identifier () const {
			return _id;
		}

		//: Consumption status.
		// Returns true when the option is consumed, meaning it is removed from
		// the argument list.
		//
		virtual bool consuming () const {
			return _consuming;
		}

		//: Fire the option.
		// This function actually handles the option. It could do whatever it wants to
		// process the option. It gets the instance to configure, the option name, the argument
		// list and an iterator pointing to the first element <I>behind the option identifier</I>.
		// If the option handler consumes further elements from the argument list, it must remove
		// them from the list and return the new head of the rest arguments as return value.
		// Otherwise the value of the <TT>rest</TT> parameter must be returned.
		//
		virtual ArgVector::iterator fireOption (C& inst, const string& id, 
							ArgVector& args, ArgVector::iterator rest) {
			return rest;
		}

};


//: Option handler adapter template.
// This template creates an option handler for instances of class C by
// using a parameterless member function that return a value of type R.
// Also a template OptionHandlerAdap1 exists that could be used for 
// member functions with one parameter.
// <P>
// Instead of explicitly instantiating the adapter templates,
// a convenience function exists that creates an option handler from a 
// member function on the fly:
// <P>
// <TT><PRE>
//template&lt;typename C, typename R&gt; 
//      inline OptionHandler&lt;C&gt;* option (const string&amp; id, bool consuming, R (C::*func) ());
//
//template&lt;typename C, typename R, typename A&gt; 
//      inline OptionHandler&lt;C&gt;* option (const string&amp; id, bool consuming, R (C::*func) (A), A arg);
// </PRE></TT>
//
template<typename C, typename R> 
class OptionHandlerAdap0 : public OptionHandler<C> {

	typedef R (C::*MemFunc0) ();

	private:
		MemFunc0	_func;

	public:
		//: Creates an option handler adapter instance.
		// If an option with name id is found in the argument list,
		// member function func will be used to handle the option.
		//
		OptionHandlerAdap0 (const string& id, bool consuming, MemFunc0 func) 
			: OptionHandler<C> (id, consuming), _func (func) {
		}

		//: An option is handled.
		// When an option id is found, member function func is applied to the inst of class C.
		// No further parameters are consumed, so rest is returned unchanged.
		//
		virtual ArgVector::iterator fireOption (C& inst, const string& id, 
							ArgVector& args, ArgVector::iterator rest) {
			(inst.*_func) ();
			return rest;
		}
};

template<typename C, typename R> 
inline OptionHandler<C>* option (const string& id, bool consuming, R (C::*func) ()) {
	return new OptionHandlerAdap0<C,R> (id, consuming, func);
}


template<typename C, typename R, typename A> 
class OptionHandlerAdap1 : public OptionHandler<C> {

	typedef R (C::*MemFunc1) (A);

	private:
		MemFunc1	_func;
		A		_arg;

	public:
		OptionHandlerAdap1 (const string& id, bool consuming, MemFunc1 func, A arg) 
			:  OptionHandler<C> (id, consuming), _func (func), _arg (arg) {
		}

		virtual ArgVector::iterator fireOption (C& inst, const string& id, 
							ArgVector& args, ArgVector::iterator rest) {
			(inst.*_func) (_arg);
			return rest;
		}
};

template<typename C, typename R, typename A> 
inline OptionHandler<C>* option (const string& id, bool consuming, R (C::*func) (A), A arg) {
	return new OptionHandlerAdap1<C,R,A> (id, consuming, func, arg);
}


//: A set of options for a configurable class C.
// This template creates a option set class for options handlers that
// use member functions of class C to process configuration options.
//
template<typename C> class Options { 

	private:
		list<OptionHandler<C>*>	_handlers;

	public:
		//: Constructor
		// An empty option set is created.
		//
		Options () {
		}

		//: Destroys the option set
		// All contained option handlers are deleted as well.
		//
		virtual ~Options () {
			clear ();
		}

	public:
		//: Try to apply options to the given instance.
		// For each element in args that matches a contained option,
		// the according option handler is invoked with the given instance.
		//
		void apply (C& inst, ArgVector& args) {
			typename list<OptionHandler<C>*>::iterator pivot;
			for (pivot = _handlers.begin (); pivot != _handlers.end (); ++pivot) {
				const char* cs_id = ((*pivot)->identifier ()).c_str ();
				ArgVector::iterator elem = args.begin ();
				while (elem != args.end ()) {
					if (strcmp (*elem, cs_id) == 0) {
						if ((*pivot)->consuming ()) {
							elem = args.erase (elem);
						} else {
							++elem;
						}
						elem = (*pivot)->fireOption (inst, (*pivot)->identifier (), 
									     args, elem);
						break;
					} else {
						++elem;
					}
				}
			}
		}

		//: Add an option handler to the option set.
		// The option handler is added without further checks.
		//
		Options& add (OptionHandler<C>* opt) {
			_handlers.push_back (opt);
			return *this;
		}

		//: Remove an option handler.
		// If the given option handler is member of the option set, it is removed.
		// Otherwise nothing is done.
		//
		Options& remove (OptionHandler<C>* opt) {
			typename list<OptionHandler<C>*>::iterator pivot 
				= find (_handlers.begin, _handlers.end, opt);

			if (pivot != _handlers.end ()) {
				erase (pivot);
			}

			return *this;
		}

		//: Clears the option set.
		// After invoking this function, the option set is empty.
		//
		Options& clear () {
			typename list<OptionHandler<C>*>::iterator pivot;
			for (pivot = _handlers.begin (); pivot != _handlers.end (); ++pivot) {
				delete *pivot;
			}
			_handlers.clear ();

			return *this;
		}

};


} // End of namespace Scrunitizer

#endif
