/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 09.08.00
*/

#include <time.h>
#include <iomanip>
#include <algorithm>

#include <SessionReport.hh>


namespace Scrunitizer {


static const char* phaseId[] = {
	"?",
	"S",
	"X",
	"P",
	"T"
};


SessionReport::SessionReport (
	ostream&	stream,
	const string&	title,
	bool		sort,
	bool		enbld,
	bool		permitCancel
) : StreamingObserver (stream, enbld, permitCancel), _title (title), _sort (sort) {
	if (enabled ()) {
		(StreamingObserver::options ()).add (option ("-nr", true, &SessionReport::enabled, false));
	}
	(StreamingObserver::options ()).add (new LogFileOption (*this, "-rf", true, true));
}


void SessionReport::beginSession (TestSession& session) {
}


void SessionReport::endSession (TestSession& session, bool cancelled) {
	char starttbuf[128];
	char  stoptbuf[128];
	char   runtbuf[128];
	time_t starttime = session.getStartTime ();
	strftime (starttbuf, 127, "%Y.%m.%d %H:%M:%S", localtime (&starttime));
	time_t stoptime = session.getStopTime ();
	strftime (stoptbuf, 127, "%Y.%m.%d %H:%M:%S", localtime (&stoptime));
	time_t runtime;
	long   runusec;
	session.getRunTime (runtime, runusec);
	strftime (runtbuf, 127, "%H:%M:%S", gmtime (&runtime));

	log () 	<< _title << endl
		<< endl
		<< " Started: " << starttbuf << endl
		<< " Stopped: " << stoptbuf  << endl
		<< "Run-Time: " << runtbuf << "." << setw (3) << setfill ('0') << runusec / 1000 <<  endl
		<< setfill (' ') << endl;

	if (session.noErrors () > 0 || session.noFailures () > 0) {
		vector<TestExceptionRecord*> records (session.getTestExceptions ());

		if (_sort) {
			sort (records.begin (), records.end (), less);
		}

		vector<TestExceptionRecord*>::iterator pivot;
		TestException* ex;	
		Test* ts;	

		int maxfnamelen = 2;
		int maxlnolen   = 1;
		int l;
		int n;
		for (pivot = records.begin (); pivot != records.end (); ++pivot) {
			ex = (*pivot)->getException ();
			if (ex->getLine () >= 0) {
				if ((l = (ex->getFileName ()).length ()) > maxfnamelen) maxfnamelen = l;
				l = 1;
				n = ex->getLine (); while (n >= 10) {n /= 10; ++l;}
				if (l > maxlnolen) maxlnolen = l;
			}
		}

		log ()	<< endl;
		for (pivot = records.begin (); pivot != records.end (); ++pivot) {
			ex = (*pivot)->getException ();
			ts = (*pivot)->getOrigin ();

			if (ex->getLine () >= 0) {
				log ()	<< setw (maxfnamelen) << (ex->getFileName ()).c_str () << " " 
					<< setw (maxlnolen)   << ex->getLine ()     << " ";
			} else {
				log ()	<< setw (maxfnamelen) << "--" << " "
					<< setw (maxlnolen)   << "-"  << " ";
			}
			log ()	<< (ex->isError () ? "E " : "f ") 
				<< (ts->isTestCase () ? " " : "S")
				<< phaseId[(int)(*pivot)->getPhase ()] << ": " 
				<< ts->getName () << ":" << endl
				<< setw (maxfnamelen + maxlnolen + 2 + 2 + 3 + 1) << " " 
				<< ex->getMessage () << endl;
		}

		log ()	<< endl
			<< "Exceptions:" << endl
			<< "  Failures.: " << session.noFailures () << endl
			<< "  Errors...: " << session.noErrors () << endl
			<< "Tests:" << endl
			<< "  Passed...: " << session.noPassed () << endl
			<< "  Failed...: " << session.noFailed () << endl
			<< "  Executed.: " << session.noExecuted () << endl
			<< "  Skipped..: " << session.noSkipped () << endl
			<< "  Total....: " << session.noEncountered () << endl
			<< endl
			<< ">>> FAILED <<<" << endl
			<< endl;
	} else {
		log ()	<< endl
			<< "Tests: " << session.noExecuted () << endl
			<< endl
			<< ">>> OK <<<" << endl
			<< endl;
	}

	log ()	<< endl;
}


void SessionReport::beginTest (Test& test, TestSession& session) {
}


void SessionReport::endTest (Test& test, TestSession& session, bool success) {
}


void SessionReport::testSkipped (Test& test, TestSession& session) {
}


void SessionReport::logException (TestExceptionRecord& exrec, TestSession& session) {
}


} // End of namespace Scrunitizer
