/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 30.07.00
*/

#ifndef _SESSION_REPORT_H_
#define _SESSION_REPORT_H_


#include <iostream>

#include <StreamingObserver.hh>


namespace Scrunitizer {


//: Generates a final test report.
// A test reporter is an observer that does its jobs <I>after</I> all the
// tests have been run. At the end of a test session, it generates
// a test report that lists all the test exceptions (failures and errors)
// which occurred during the session. Finally, a test session summary and
// the verdict, OK or FAILED, is logged.
// <P>
// The session report is designed so that it could be combined with a 
// BriefSessionLog or a VerboseSessionLog.
// <P>
// See also <A HREF="observerdemo.html#report">observer output examples</A>
//
class SessionReport : public StreamingObserver {

	private:
		string		_title;
		bool		_sort;

	public:
		//: Creates a test report observer.
		// The title is printed at the beginning of the test session.
		// If <TT>sort</TT> is true, all test exceptions are sorted before they
		// are printed at the end of the test session.
		//
		// Session reports assign default options due to the following rule:<BR>
		// <I>If the report is enabled when this function is called</I>, an option handler for
		// the flag "-nr" (no report) is assigned. That <I>disables</I> the session report
		// and consumes the flag.<BR>
		// Otherwise, an file option handler for "-rf" (report file) is assigned, that
		// sets the output file and consumes the flag.
		// <P>
		// This behaviour allows to associate two session report observers to the 
		// test manager:
		// <UL>
		// <LI>One that produces an screen output per default (enabled = true),
		// <LI>and another one that writes the report in a file on demand (enabled = false).
		// </UL>
		//
		SessionReport (
			ostream&	stream,
			const string&	title,
			bool		sort = false,
			bool		enabled = true,
			bool		permitCancel = true
		);

		//: Destructor.
		// Does nothing extra.
		//
		virtual ~SessionReport () {}

	public:
		//: A new session has been started.
		// A header with the session title is generated and start, stop and 
		// run time are logged.
		//
		virtual void beginSession (TestSession& session);

		//: The session has come to an end.
		// All test exceptions, errors and failures, are printed (if any).
		// If sorting is enabled, the list of exception is sorted
		// before it is printed. A test session statistics is 
		// generated that summarises the test session outcome. 
		// Finally, the verdict of the session is printed.
		//
		virtual void endSession   (TestSession& session, bool cancelled);

		//: A new test starts.
		// Nothing is done.
		//
		virtual void beginTest (Test& test, TestSession& session);

		//: The test is finished.
		// Nothing is done.
		//
		virtual void endTest   (Test& test, TestSession& session, bool success);

		//: A test was skipped.
		// Nothing is done.
		//
		virtual void testSkipped (Test& test, TestSession& session);

		//: A test exception was thrown.
		// Nothing is done. All test exceptions are summarized at
		// the end of the session.
		//
		virtual void logException (TestExceptionRecord& mr, TestSession& session);
};


} // End of namespace Scrunitizer

#endif
