/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 19.08.00
*/

#ifndef _LOGGING_TEST_OBSERVER_H_
#define _LOGGING_TEST_OBSERVER_H_


#include <iostream.h>
#include <fstream.h>

#include <TestObserver.hh>


namespace Scrunitizer {


//: Exception class used when a log file cannot be opened.
// This exception is thrown when subclasses of StreamingObserver
// are configured.
//
class CannotOpenLog { 
	private:
		string _fileName;

	public:
		//: Creates a new exception instance.
		// Also the name of the file that could not be opened
		// must be given.
		//
		CannotOpenLog (
			const string& fileName
		) : _fileName (fileName) {
		}

		//: The name of the file that could not be opened.
		// Reader method.
		//
		const string& getFileName () {
			return _fileName;
		}
};


//: Observer that generates a log by writing to a stream.
// Logging test observers are typically used in terminal environments for
// test log output or for generating reports as text files. This class is
// the basis for a variety of log generators that create terminal and
// file logs.
//
class StreamingObserver : public TestObserver {

	private:
		Options<StreamingObserver>	_options;
		bool				_enabled;
		bool				_permitCancel;
		ostream*			_log;
		ofstream*			_file;

	public:
		//: Create a logging observer.
		// A reference to a stream is needed to which the log is written
		// as long as the stream isnt changed. If <TT>enabled</TT> is true, the
		// observer will produce a log. If <TT>permitsCancel</TT> is true, the observer
		// allows to cancel a test session.
		//
		StreamingObserver (
			ostream& log,
			bool enabled = true,
			bool permitCancel = false
		);

		//: The destructor.
		// The log stream will be closed if the log was set by
		// specifying a filename.
		//
		virtual ~StreamingObserver ();

	private:
		/* Prevent using the copy-constructor and the assignment operator. */
		StreamingObserver (const StreamingObserver&);
		void operator= (const StreamingObserver&);

	public:
		//: Configure the log.
		// True is returned when the log is enabled.
		// Note that during configuration of an observer, exceptions
		// could be thrown, i.e. CannotOpenLog.
		//
		virtual bool configureFrom (ArgVector& args);

		//: Get the defined set of options.
		// Only an empty set of options is returned.
		// StreamingObserver itself does not have any option handlers.
		//
		virtual Options<StreamingObserver>& options () {
			return _options;
		}

		//: Is the observer enabled?
		// Returns the enabled-status.
		//
		virtual bool enabled () const {
			return _enabled;
		}

		//: Change the enabled-status.
		// If set to false before <TT>configureFrom</TT> is called, the observer
		// will not produce a log.
		//
		virtual void enabled (bool state);


		//: Check if the observer permits cancelling the execution of tests.
		// Return the value of <TT>permitCancel</TT>.
		//
		virtual bool permitsCancel () const {
			return _permitCancel;
		}

		//: Change the value of permitCancel.
		// If set to false, always a complete log will be written because
		// cancelling the test session is not possible anymore. Do this only
		// if absolutely necessary.
		//
		virtual void permitCancel (bool state);

		//: Set another log output stream.
		// This function could be used to change the log output
		// after the observer was created. If the current log stream
		// was set by specifying a file name, the current log is closed
		// before the new one is set.
		//
		virtual void setLog (ostream& log);

		//: Create a file log.
		// The observer will write its log into a file. If creating a file
		// with this name in the current environment is not possible,
		// the exception CannotOpenLog is thrown. The log file is closed
		// when another log is set or when the observer is destroyed.
		//
		virtual void setLog (const string& filename);

		//: Returns a reference to the log output stream.
		// Convenience function.
		//
		ostream& log () {
			return *_log;
		}

};


typedef Options<StreamingObserver> ObserverOptions;

//: Exception class.
// Thrown when a name for the log file was missing behind the option
// for using a log file.
//
class LogFileNameMissing { 
	private:
		string	_id;

	public:
		//: Create a new exception instance.
		// The option identifier must be specified.
		//
		LogFileNameMissing (
			const string& id
		) : _id (id) {
		}

		//: The identifier of the option that caused the problem.
		// Reader method.
		// 
		const string& getOptionId () {
			return _id;
		}
};


//: Option handler for log file names.
// This option handler interprets the next argument (if it does <I>not</I>
// begin with a dash) as a file name and sets the according file as
// log output stream.
//
class LogFileOption : public OptionHandler<StreamingObserver> {

	private:
		StreamingObserver&	_observer;
		bool			_autoEnable;

	public:
		//: Creates a log file option handler instance.
		// The log file (the argument behind the argument equal to id) is associated
		// to the given observer. If <TT>autoEnable</TT> is true, the observer will also
		// be enabled as well.
		// It is strongly recommended that
		// the option flag is consumed because the file
		// argument will be consumed anyway.
		//
		LogFileOption (
			StreamingObserver& observer,
			const string& id, 
			bool autoEnable = true,
			bool consuming = true
		); 

		//: The destructor.
		// Does nothing special.
		//
		virtual ~LogFileOption ();

	public:
		//: Handles a log file option.
		// The argument behind the option flag is interpreted as file name.
		// (If it does <I>not</I> begin with a dash!) It is tried to create
		// a file with this name in the current environment (working directory).
		// If the no file name is present, a LogFileNameMissing exception is thrown.
		// If the file cannot be opened, a CannotOpenLog exception is thrown by
		// <TT>setLog</TT> of StreamingObserver.
		//
		virtual ArgVector::iterator fireOption (
			StreamingObserver& inst, 
			const string& id, 
			ArgVector& args, 
			ArgVector::iterator rest
		);

};



} // End of namespace Scrunitizer

#endif
