/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 05.08.00
*/

#ifndef _TEST_H_
#define _TEST_H_

#include <iostream>
#include <string>

#include <TestConductor.hh>


namespace Scrunitizer {

//: Thrown if a test is not properly set-up
// This exception has no contents. It is thrown only during construction
// of test cases or test suites and indicates that some part of the tests
// configuration is set to an illegal value (i.e. a NULL pointer).
//
struct IllegalTestSetUp {};


//: Abstract interface for all kinds of unit tests.
// This abstract class defines all the operations which a test must support to
// be used for unit testing. The only attribute it defines
// is the name of the test.
//
class Test {

	private:
		string	_name;

	public:
		//: Basic constructor. 
		// Only sets the name.
		//
		Test (const string& name) : _name (name) { }

		//: Destructor.
		// Ensures that all derived classes have a virtual destructor.
		virtual ~Test () { }

	public:
		//: Returns the name of the test.
		// The name of the test could only be set when an test object
		// is created. It should at least be unique within a test suite.
		//
		virtual string getName () const {
			return _name;
		}

		//: Delegate the execution of the test to a test conductor.
		// This operation is used to tell the test that it should run itself
		// with the help of the given test conductor. This operation could
		// be used to prepare for execution by a conductor. Normally, this
		// operation simply calls the conductor to execute the test. <I>Unit test classes
		// that perform actual test actions should not override this method.</I>
		// It returns true if the test was successfully executed, otherwise false.
		//
		virtual bool run (TestConductor&) = 0;

		//: Set-up the test fixture.
		// This operation is called by the test conductor so that the test can
		// prepare itself for execution. If that operation fails, neither the
		// <T>test</T> operation nor the <TT>tearDown</TT> function is called!
		//
		virtual void setUp () = 0;

		//: Execute the test.
		// This method should perform the test of some application domain functionality,
		// either directly or indirectly. Test cases should do the actual testing here,
		// test suites should iterate over their contents when this method is called.
		// In case something doesn't behave as expected, an exception should be thrown.
		// Otherwise the function simply returns.
		//
		virtual void test (TestConductor&) = 0;

		//: Tear-down of the test fixture.
		// After the test was executed, this member function is called to
		// tear down the test fixture, i.e. freeing previously allocated resources.
		// This operation is executed whether the test action was successful or not.
		//
		virtual void tearDown () = 0;

		//: Check if the test is a test case.
		// Returns true when the test instance <I>itself</I> tests some application
		// domain functionality.
		//
		virtual bool isTestCase () const = 0;

		//: Returns the number of contained test cases.
		// Computes the total amount of directly or indirectly included test cases,
		// including the called instance itself.
		//
		virtual int noTestCases () const = 0;

		//: Returns the number of contained test suites.
		// Computes the total amount of directly or indirectly included test suites,
		// including the called instance itself.
		//
		virtual int noTestSuites () const = 0;


		//: Write a string representation of the test to an output stream.
		// Support function for the (non-virtual) output stream operator &lt;&lt;.
		//
		virtual void put (ostream&) const = 0;


	//: Write a test to an output stream.
	// The output operator simply calls member function <TT>put</TT> with the
	// stream as argument. Because <TT>put<TT> is virtual, each descendant of Test
	// can define its own stream output format.
	//
	// friend ostream& operator<< (ostream& stream, const Test& test);
};


} // End of namespace Scrunitizer


inline ostream& operator<< (ostream& stream, const Scrunitizer::Test& test) {
	test.put (stream);
	return stream;
}

#endif
