/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.4 / Last changed: 29.10.00
*/


#ifndef _TEST_APPLIANCE_H_
#define _TEST_APPLIANCE_H_


#include <TestCase.hh>
#include <Applicable.hh>


namespace Scrunitizer {


//: A template that creates test cases by applying functor objects to an arbitrary object.
// This template generates test cases that use applicables for <I>all phases of a unit test</I>:
// Test set-up, execution and tear-down. So a TestAppliance is an extension of a TestGadget. 
// And  also here, the fixture object itself does <I>not</I> need to inherit from TestCase. <BR>
// Most likely, all the needed applicables are created from member functions of the test fixture class.
// They are all owned by the test appliance. So when the appliance is 
// destroyed, this objects are deleted as well. 
// <P>
// The <TT>apply</TT> template function could be used
// to construct Applicable-instances out of member functions of the fly. Note that there also
// versions of the apply functions that allow to specify constant parameters for the 
// member functions.<BR>
// The test appliance itself could be easily created with the following template functions:
// <P>
// <TT><PRE>
//template&lt;class T&gt; inline TestAppliance&lt;T&gt;* makeTestAppliance (
//      const string&amp; name,
//      Applicable&lt;T&gt;* testFunc, 
//      Applicable&lt;T&gt;* setUpFunc,
//      Applicable&lt;T&gt;* tearDownFunc
//);
//
//template&lt;class T&gt; inline TestAppliance&lt;T&gt;* makeTestAppliance (
//      const string&amp; name,
//      T* object,
//      Applicable&lt;T&gt;* testFunc, 
//      Applicable&lt;T&gt;* setUpFunc,
//      Applicable&lt;T&gt;* tearDownFunc,
//      bool releaseFixture = false
//);
//
//template&lt;class T&gt; inline TestAppliance&lt;T&gt;* makeStdTestAppliance (
//      const string&amp; name,
//      Applicable&lt;T&gt;* testFunc
//);
//
//template&lt;class T&gt; inline TestAppliance&lt;T&gt;* makeStdTestAppliance (
//      const string&amp; name,
//      T* object,
//      Applicable&lt;T&gt;* testFunc, 
//      bool releaseFixture = false
//);
// </PRE></TT>
//
// The <TT>makeStdTestAppliance</TT> functions automatically use the member function <TT>setUp ()</TT> and
// <TT>tearDown ()</TT> to construct set-up and tear-down applicables, so the fixture class T must have
// such member functions. If the fixture object is given as parameter, it is assumed that the instance
// is shared between test appliances, so <TT>releaseFixture</TT> defaults to false.
//
template<class T> class TestAppliance : public TestCase {

	typedef Applicable<T>*	FuncApplication;

	private:
		T*		_testFixture;
		bool		_releaseFixture;
		Applicable<T>*	_testFunc;
		Applicable<T>*	_setUpFunc;
		Applicable<T>*	_tearDownFunc;

	public:
		//: Create a new test appliance instance.
		// The test, set-up and tear-down function applicables must be specified.
		// If <TT>releaseFixture</TT> is true, the fixture instance is owned by the 
		// appliance, so it is deleted when the test appliance is destroyed.
		//
		TestAppliance (
			const string name,
			T* fixture,
			Applicable<T>*	testFunc,
			Applicable<T>*	setUpFunc,
			Applicable<T>*	tearDownFunc,
			bool releaseFixture
		) : TestCase (name),  _testFixture (fixture), _releaseFixture (releaseFixture), 
		    _testFunc (testFunc), _setUpFunc (setUpFunc), _tearDownFunc (tearDownFunc) {
			if (testFunc == 0 || setUpFunc == 0 || tearDownFunc == 0) throw IllegalTestSetUp ();
		}

		//: Destroys the test appliance
		// Deletes the fixture object and all used applicables.
		//
		virtual ~TestAppliance () {
			if (_releaseFixture) delete _testFixture;
			delete _testFunc;
			delete _setUpFunc;
			delete _tearDownFunc;
		}

	private:
		/* Prevent using the copy-constructor and the assignment operator. */
		TestAppliance (const TestAppliance&);
		void operator= (const TestAppliance&);

	public:
		//: Construction of the test environment.
		// Calls the set-up applicable.
		// 
		virtual void setUp () {
			(*_setUpFunc) (_testFixture);
		}

		//: Execute the test.
		// Calls the test action applicable.
		//
		virtual void test () {
			(*_testFunc) (_testFixture);
		}

		//: Deconstruction of the test environment.
		// Calls the tear-down applicable.
		// 
		virtual void tearDown () {
			(*_tearDownFunc) (_testFixture);
		}

};


template<class T> inline TestAppliance<T>* makeTestAppliance (
	const string& name,
	Applicable<T>* testFunc, 
	Applicable<T>* setUpFunc,
	Applicable<T>* tearDownFunc
) {
	return new TestAppliance<T> (name, new T (), testFunc, setUpFunc, tearDownFunc);
}


template<class T> inline TestAppliance<T>* makeTestAppliance (
	const string& name,
	T* object,
	Applicable<T>* testFunc, 
	Applicable<T>* setUpFunc,
	Applicable<T>* tearDownFunc,
	bool releaseFixture = false
) {
	return new TestAppliance<T> (name,  object, testFunc, setUpFunc, tearDownFunc, releaseFixture);
}


template<class T> inline TestAppliance<T>* makeStdTestAppliance (
	const string& name,
	Applicable<T>* testFunc
) {
	return new TestAppliance<T> (name, new T (), testFunc, apply (&T::setUp), apply(&T::tearDown), 
				     true);
}


template<class T> inline TestAppliance<T>* makeStdTestAppliance (
	const string& name,
	T* object,
	Applicable<T>* testFunc, 
	bool releaseFixture = false
) {
	return new TestAppliance<T> (name, object, testFunc, apply (&T::setUp), apply(&T::tearDown), 
				     releaseFixture);
}


} // End of namespace Scrunitizer
#endif
