/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 30.07.00
*/

#ifndef _TEST_CASE_H_
#define _TEST_CASE_H_

#include <Test.hh>


namespace Scrunitizer {

//: Base implementation of a test case.
// This class is intended to be used as base class for "real"
// test cases. It provides a default implementation of the required
// functionality of a test case.
//
class TestCase : public Test {

	public:
		//: The constructor.
		// Only the name is set.
		//
		TestCase (const string& name) : Test (name) {}

	public:
		//: Executes the test case with the help of the given test conductor.
		// Simply delegates testing to the execute method of the test conductor
		// so that set-up, test and tear-down functions are called while ensuring
		// a well defined behaviour.
		//
		virtual bool run (TestConductor& conductor) {
			return conductor.execute (*this);
		}

		//: Construction of the test fixture.
		// Dummy implementation. Has no functionality.
		// 
		virtual void setUp () {}

		//: Perform the test activity.
		// The only thing this function does is to call the parameterless test method 
		// (see below) to execute the test of application domain functionality.
		//
		virtual void test (TestConductor&) {
			test ();
		}

		//: Execute the test.
		// In case something doesn't behave as expected, a TestFailure should be thrown.
		// Otherwise the function simply returns.
		//
		virtual void test () = 0;

		//: Deconstruction of the test fixture.
		// Dummy implementation. Has no functionality.
		// 
		virtual void tearDown () {}

		//: Tests if an instance is a test case.
		// Returns always true.
		// 
		virtual bool isTestCase () const {
			return true;
		};

		//: Counts the number of (directly or indirectly) contained test cases.
		// Returns always 1.
		//
		virtual int noTestCases () const {
			return 1;
		};

		//: Counts the number of (directly or indirectly) contained test suites.
		// Returns always 0.
		//
		virtual int noTestSuites () const {
			return 0;
		};

		//: Output the test case to the stream.
		//  The string "Test case: " and the name is printed.
		// 
		virtual void put (ostream& stream) const {
			stream << "Test case: " << getName ();
		}
};


} // End of namespace Scrunitizer

#endif
