/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.4 / Last changed: 29.10.00
*/


#ifndef _TEST_CASE_ADAPTER_H_
#define _TEST_CASE_ADAPTER_H_


#include <TestCase.hh>
#include <Binder.hh>


namespace Scrunitizer {


inline void noop () {}

typedef void (*noop_type) ();


//: Constructs test Cases from functions.
// The test case adapter template creates a test case class from functions for the test activity, 
// test fixture set-up and tear-down. Only functors could be used <I>directly</I>, that means plain
// functions have take no arguments. If a function should be used that requires some parameters, a binder
// could be used to associate static values to parameters (see Binder0).<BR>
// A test case adapter instance could be easily created by using the convenience function
// <TT>makeTestCase</TT>. This function takes one to three functor arguments for the test action, 
// and for fixture set-up and tear-down. If no functor is given for set-up and tear-down, a default
// no-operation function is used.
// <P>
// <TT><PRE>
//template&lt;typename T, typename S, typename D&gt;
//inline TestCaseAdapter&lt;T,S,D&gt;* makeTestCase (
//      const string&amp; name,
//      T testFunc, 
//      S setUpFunc,
//      D tearDownFunc
//);
//
//template&lt;typename T, typename S&gt; 
//inline TestCaseAdapter&lt;T,S,noop_type&gt;* makeTestCase (
//      const string&amp; name,
//      T testFunc, 
//      S setUpFunc
//);
//
//template&lt;typename T&gt; 
//inline TestCaseAdapter&lt;T,noop_type,noop_type&gt;* makeTestCase (
//      const string&amp; name,
//      T testFunc
//);
// </PRE></TT>
//
template<typename T, typename S, typename D> class TestCaseAdapter : public TestCase {

	private:
		T	_testFunc;
		S	_setUpFunc;
		D	_tearDownFunc;

	public:
		//: Create a new test adapter instance.
		// In addition to the name, a test function of type T, a set-up function
		// of type S and a tear-down function of type D have to be specified.
		//
		TestCaseAdapter(const string& name,
				T testFunc,
				S setUpFunc,
				D tearDownFunc
		) : 	TestCase (name), 
			_testFunc (testFunc), 
			_setUpFunc (setUpFunc), 
			_tearDownFunc (tearDownFunc) {
		}

	public:
#ifdef HPUX
		//: Run the test.
		// Simply calls <TT>test ()</TT>.
		//
		virtual void test (TestConductor&) {
			test ();
		}
#endif
		//: Fixture set-up.
		// Invokes the set-up function.
		//
		virtual void setUp () {
			_setUpFunc ();
		}

		//: Do the test.
		// Invokes the test function.
		//
		virtual void test () {
			_testFunc ();
		}

		//: Fixture tear-down.
		// Invokes the tear-down function.
		//
		virtual void tearDown () {
			_tearDownFunc ();
		}

};


template<typename T, typename S, typename D> 
inline TestCaseAdapter<T,S,D>* makeTestCase (
	const string& name,
	T testFunc, 
	S setUpFunc,
	D tearDownFunc
) {
	return new TestCaseAdapter<T,S,D> (name, testFunc, setUpFunc, tearDownFunc);
}


template<typename T, typename S> 
inline TestCaseAdapter<T,S,noop_type>* makeTestCase (
	const string& name,
	T testFunc, 
	S setUpFunc
) {
	return new TestCaseAdapter<T,S,noop_type> (name, testFunc, setUpFunc, noop);
}


template<typename T> 
inline TestCaseAdapter<T,noop_type,noop_type>* makeTestCase (
	const string& name,
	T testFunc
) {
	return new TestCaseAdapter<T,noop_type,noop_type> (name, testFunc, noop, noop);
}


} // End of namespace Scrunitizer
#endif
