/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 30.07.00
*/

#ifndef _TEST_CONDUCTOR_H_
#define _TEST_CONDUCTOR_H_


#include <Callable.hh>


namespace Scrunitizer {


class Test; // Forward reference


//: Interface to be used by tests for handling their execution.
// Concept of an item that executes unit test according to
// a well-defined semantics. A test conductor is handed a test 
// - or a test suite - so that it can run trough all
// three phases of test: Fixture set-up, test execution and fixture tear-down.
//
class TestConductor {

	public:
		//: Initialise a test conductor.
		// Does nothing.
		//
		virtual ~TestConductor () {}

	public:
		//: Execute the given test object.
		// The conductor first calls the set-up method of the given test.
		// If that fails, test execution and fixture tear-down are skipped. If setting up
		// the test fixture was successful, the test method is called. Regardless if that was
		// successful or not (an exception was thrown), the tear-down function of
		// the test is called. Any exceptions thrown by the tear-down function
		// have no further effects.<BR>
		// Note that a real test conductor does a lot
		// of other things while executing a test, like result collection and logging,
		// translating arbitrary application domain exception in test failures,
		// maintaining a user interface, etc. The method returns true if execution of the
		// test was successful, otherwise false.
		//
		virtual bool execute (Test&) = 0;


		//: Tell the conductor that this test was skipped.
		// Also when test are skipped (i.e. because a previous test failed in a test chain)
		// the conductor must be informed. That is necessary to be able to do bookkeeping, make
		// log entries or for notifying the GUI.
		//
		virtual void skip (Test&) = 0;

};


} // End of namespace Scrunitizer

#endif
