/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 19.08.00
*/

#ifndef _TEST_EXCEPTION_RECORD_H_
#define _TEST_EXCEPTION_RECORD_H_


#include <Test.hh>
#include <TestPhase.hh>
#include <TestExceptions.hh>


namespace Scrunitizer {

//: Record for a test exception
// This class groups together all items that were involved
// when an exception occured during test execution.
//
class TestExceptionRecord {
	
	private:
		Test*		_origin;
		TestException*	_exception;
		TestPhase	_phase;
		bool		_release;

	public:
		//: Create a new test exception record.
		// As input is needed the test which has caused the
		// exception, the test exception itself 
		// and the test phase in which the exception was thrown. If <TT>release</TT>
		// is set to true, the exception is deleted when this record is destroyed.
		//
		TestExceptionRecord (
			Test*		origin,
			TestException*	exception,
			TestPhase	phase = UNKNOWN,
			bool		release = true
		);

		//: The destructor.
		// If <TT>release</TT> is true, also the exception is deleted.
		//
		virtual ~TestExceptionRecord ();

	private:
		/* Prevent using the copy-constructor and the assignment operator. */
		TestExceptionRecord (const TestExceptionRecord&);
		void operator= (const TestExceptionRecord&);

	public:
		//: Get the test that caused the exception.
		// Simple reader method.
		//
		virtual Test* getOrigin () const {
			return _origin;
		}

		//: Get the exception that is stored in the record.
		// Simple reader method.
		//
		virtual TestException* getException () const {
			return _exception;
		}

		//: Get the phase the test was in when the exception occurred.
		// Simple reader method.
		//
		virtual TestPhase getPhase () const {
			return _phase;
		}

		//: Check if the records contains an error.
		// Returns true when the contained test exception is a test error
		// (and not only a test failure).
		//
		virtual bool isErrorRecord () const {
			return _exception->isError ();
		}

	//: Function to compare two records.
	// This function could be used as comparator in STL algorithms.
	// Its primary sort criteria is the name of the originating test,
	// the second is the exception type (error of failure) and the third
	// sort criteria is the exception message.
	//
	friend bool less (TestExceptionRecord* lhs, TestExceptionRecord* rhs);
};

bool less (TestExceptionRecord* lhs, TestExceptionRecord* rhs);


} // End of namespace Scrunitizer

#endif
