/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version  / Last changed: 19.08.00
*/


#ifndef _TEST_EXCEPTIONS_H_
#define _TEST_EXCEPTIONS_H_

#include <string>


namespace Scrunitizer {

#define UNKNOWN_FILE	"<unknown>"
#define UNKNOWN_LINE	(-1)


//: Base class for unit test exceptions.
// This class primarily servers as base class for TestFailure and TestException.
// It provides the basic mechanism for handling unit test exception.
// It is implemented with the handle-body-idiom: The actual exception
// data is stored inside an internal data structure. That allows a very fast
// implementation of the copy-constructor and the assignment operator. This is
// essential for all descendants of this class because they might be caught and re-thrown
// many times. 
//
class TestException {

	class ExceptionInfo {
		int		_refCount;
		string		_message;
		string		_fileName;
		int		_line;

		ExceptionInfo (const string& message, const string& fileName, int line);
		~ExceptionInfo ();

		friend class TestException;
	};

	private:
		ExceptionInfo*	_body;

	protected:
		//: Initialise the exception with a descriptive message
		// The message should describe the exception cause.
		// In addition, the name of the source file in which the test is
		// defined and the line number could be specified. If they are
		// unknown, symbolic default values are used.
		//
		TestException (
			const string& message,
			const string& fileName	= UNKNOWN_FILE,
			int line 		= UNKNOWN_LINE
		 );

		//: The copy-constructor
		// Creates only a reference to the internal, shared data structure.
		//
		TestException (const TestException& te);

	public:
		//: Virtual destructor
		// Releases its reference to the internal data structure. Only when this
		// was the last reference, all the data is deleted as well.
		//
		virtual ~TestException ();

		//: Assignment operator
		// Using this operator is cheap because only the exception body
		// is reused.
		//
		virtual TestException& operator= (const TestException& rhs);

	public:
		//: Get the message.
		// Simple reader method.
		//
		virtual string getMessage () const {
			return _body->_message;
		}

		//: Return the exception source file name.
		// The name of the file that contains the code that has generated
		// the test exception. The real file name is only available if it
		// was specified when the exceptions constructor was called. Otherwise
		// the value UNKNOWN_FILE is returned.
		//
		virtual string getFileName () const {
			return _body->_fileName;
		}

		//: Return the line number in the exception source file name.
		// The line number of the place in the code that has generated
		// the test exception. The real line number is only available if it
		// was specified when the exceptions constructor was called. Otherwise
		// the value UNKNOWN_LINE is returned.
		//
		virtual int getLine () const {
			return _body->_line;
		}

		//: Test if this exception is an error.
		// An exception is an error if it was raised because something unforseen has happened.
		// Normal negative test outcomes are represented as failures, not errors!
		//
		virtual bool isError () const = 0;
};


//: A test failure represents a negative test outcome. 
// That means that something that was tested
// on purpose has produced a negative test result. So TestFailures are
// generated when application domain functionality is tested.
//
class TestFailure : public TestException {

	public:
		//: Initialise the failure with a descriptive message.
		// Relies on the constructor of TestException.
		//
		TestFailure (
			const string& message,
			const string& fileName	= UNKNOWN_FILE,
			int line 		= UNKNOWN_LINE
		) : TestException (message, fileName, line) {}

		//: The copy-constructor
		// Relies on the copy-constructor of TestException.
		//
		TestFailure (const TestFailure& tf) : TestException (tf) {}

	public:
		//: Test if this exception is an error.
		// Retruns always false, because this is a failure, not an error.
		//
		virtual bool isError () const {
			return false;
		}
};


//: A test error represents an unexpected situation during execution of unit tests.
// Test errors are generated when something fails in an unexpected way. That especially includes
// test fixture set-up and tear-down. Whenever the framework catches an exception that is not
// already a TestException, it generates a test error.
//
class TestError : public TestException {

	public:
		//: Initialise the error with a descriptive message.
		// Relies on the constructor of TestException.
		//
		TestError (
			const string& message,
			const string& fileName	= UNKNOWN_FILE,
			int line 		= UNKNOWN_LINE
		) : TestException (message, fileName, line) {}

		//: The copy-constructor
		// Relies on the copy-constructor of TestException.
		//
		TestError (const TestError& te) : TestException (te) {}

	public:
		//: Test if this exception is an error.
		// Returns always true.
		//
		virtual bool isError () const {
			return true;
		}
};


} // End of namespace Scrunitizer

#endif

