/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.4 / Last changed: 29.10.00
*/


#ifndef _TEST_GADGET_H_
#define _TEST_GADGET_H_


#include <TestCase.hh>
#include <Applicable.hh>


namespace Scrunitizer {


//: A template that creates test cases by applying a functor to a test fixture object.
// This template generates test cases that use an applicable for test execution.
// That applicable is most likely generated from a member function of the test fixture class.
// Note that the fixture class does <I>not</I> need to inherit from Test. <BR>
// The fixture object and the applicable instance are owned by the test gadget. So when the 
// gadget is destroyed, this objects are deleted as well.
// The template function <TT>apply</TT> could be used to construct the Applicable-instance for the test 
// action on the fly (see Applicable). 
// Note that there also versions of the apply functions that allow to specify 
// constant parameters for the member function. The test gadget itself could be easily created 
// with the <TT>makeTestGadget</TT> template functions:
// <P>
// <TT><PRE>
//template&lt;class T&gt; inline TestGadget&lt;T&gt;* makeTestGadget (
//      const string&amp; name,
//      Applicable&lt;T&gt;* testFunc
//);
//
//template&lt;class T&gt; inline TestGadget&lt;T&gt;* makeTestGadget (
//      const string&amp; name,
//      T* object,
//      Applicable&lt;T&gt;* testFunc,
//      bool releaseFixture = false
//);
// </PRE></TT>
//
// The first version constructs automatically a fixture instance which is owned by the
// gadget (so it is also deleted when the gadget is destroyed). The second version allows
// to specify the fixture object. Because such a function is needed when a fixture is
// used for multiple gadgets, the <TT>releaseFixture</TT> parameter defaults to false.
//
template<class T> class TestGadget : public TestCase {

	private:
		T*		_testFixture;
		Applicable<T>*	_testFunc;
		bool		_releaseFixture;

	public:
		//: Create a test gadget from a fixture instance and an applicable.
		// A new test case is created from combining a test fixture object (of type T)
		// with an applicable (which is usable with type T). If <TT>releaseFixture</TT>
		// is true, the fixture instance will be deleted when the test gadget is destroyed.
		//
		TestGadget (
			const string name,
			T* fixture,
			Applicable<T>*	testFunc,
			bool releaseFixture
		) : TestCase (name), _testFixture (fixture), _testFunc (testFunc), 
		                     _releaseFixture (releaseFixture) {
			if (testFunc == 0) throw IllegalTestSetUp ();
		}

		//: Destroys the test gadget.
		// Also the test fixture object and the applicable instance are deleted.
		//
		virtual ~TestGadget () {
			delete _testFunc;
			if (_releaseFixture) delete _testFixture;
		}

	private:
		/* Prevent using the copy-constructor and the assignment operator. */
		TestGadget (const TestGadget&);
		void operator= (const TestGadget&);

	public:
		//: Execute the test.
		// The test function Applicable instance is applied to the test fixture
		// instance. In practice that means that the member function is applied
		// to the fixture object.
		//
		virtual void test () {
			(*_testFunc) (_testFixture);
		}
};


template<class T> inline TestGadget<T>* makeTestGadget (
	const string& name,
	Applicable<T>* testFunc
) {
	return new TestGadget<T> (name, new T (), testFunc, true);
}


template<class T> inline TestGadget<T>* makeTestGadget (
	const string& name,
	T* object,
	Applicable<T>* testFunc,
	bool releaseFixture = false
) {
	return new TestGadget<T> (name,  object, testFunc, releaseFixture);
}


} // End of namespace Scrunitizer
#endif
