/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.4 / Last changed: 30.10.00
*/


#include <algorithm>
#include <exception>

#include <TestManager.hh>


namespace Scrunitizer {


void TestManager::ExecutionGate::call () {
	switch (_phase) {
		case UNKNOWN:
			break;
		case SET_UP:
			_test->setUp ();
			break;
		case EXECUTE:
			_test->test (*_manager);
			break;
		case TEAR_DOWN:
		case PREMATURE_TEAR_DOWN:
			_test->tearDown ();
			break;
	}
}



TestManager::TestManager (bool recordExceptions)
 : _execGate (this), 
   _baseCallable (&_execGate), 
   _executed (0), 
   _failed (0), 
   _skipped (0), 
   _failures (0), 
   _errors (0), 
   _testExceptions (), 
   _recordExceptions (recordExceptions) {
}


TestManager::~TestManager () {
	reset ();

	list<CallGate*>::iterator pivot;
	for (pivot = _callGates.begin (); pivot != _callGates.end (); ++pivot) {
		delete *pivot;
	}	

	list<TestFixture*>::iterator fixture;
	for (fixture = _fixtures.begin (); fixture != _fixtures.end (); ++fixture) {
		delete *fixture;
	}	
}


void TestManager::reset () {
	_executed	= 0;
	_failed		= 0;
	_skipped	= 0;
	_failures	= 0;
	_errors		= 0;

	vector<TestExceptionRecord*>::iterator pivot;
	for (pivot = _testExceptions.begin (); pivot != _testExceptions.end (); ++pivot) {
		delete *pivot;
	}
	_testExceptions.clear ();
}


bool TestManager::executePhase (Test& test, TestPhase phase) {
	_execGate._test  = &test;
	_execGate._phase = phase;
 
	bool success = false;
	try {
		_baseCallable->call ();
		success = true;
	}
	catch (TestFailure& failure) {
		++_failures;
		logException (&test, new TestFailure (failure), phase);
	}
	catch (TestError& error) {
		++_errors;
		logException (&test, new TestError (error), phase);
	}
	catch (StopTesting& ex) {
		// Handled in execute, so rethrow it.
		throw;
	}
	catch (exception &ex) {
		++_errors;
		logException (&test, new TestError (ex.what ()), phase);
	}		
	catch (...) {
		++_errors;
		logException (&test, new TestError ("Uncaught exception"), phase);
	}

	return success;
}


bool TestManager::execute (Test& test) {
	bool success;

	_execGate._test  = &test;
	_execGate._phase = UNKNOWN;

	beginTest (test);

	try {
		if (success = executePhase (test, SET_UP)) {
			if (success = executePhase (test, EXECUTE)) {
				success = executePhase (test, TEAR_DOWN);
			} else {
				executePhase (test, PREMATURE_TEAR_DOWN);
			}
		}
	}
	catch (StopTesting& ex) {
		// Clean up and re-throw...
		throw;
	}

	if (! success) {
		checkForSkippedTests (test);
	}
	if (test.isTestCase ()) {
		++_executed;
		if (! success) ++_failed;
	}

	endTest (test, success);

	if (_stop) {
		throw StopTesting ();
	}

	return success;
}


void TestManager::skip (Test& test) {
	_skipped += test.noTestCases ();
	testSkipped (test);
}


void TestManager::addCallGate (CallGate* gate) {
	if (gate == 0) {
		throw IllegalTestSetUp ();
	}

	gate->setTarget (_baseCallable, false);
	_baseCallable = gate;

	_callGates.push_front (gate);
}


void TestManager::removeCallGate (CallGate* gate) {
	if (gate == 0) return;

	list<CallGate*>::iterator pivot;
	for (pivot = _callGates.begin (); pivot != _callGates.end (); ++pivot) {
		if (*pivot == gate) {
			if (pivot == _callGates.begin ()) {
				_baseCallable = (*pivot)->getTarget ();
			} else {
				list<CallGate*>::iterator pred = --pivot;
				(*pred)->setTarget ((*pivot)->getTarget ());
			}
			_callGates.erase (pivot);
			break;
		}
	}
}


void TestManager::conduct (Test& test) {
	reset ();

	bool cancelled = false;
	struct timezone tz;

	_stop = false;

	gettimeofday (&_startTime, &tz);
	beginSession ();

	try {
		test.run (*this);
	}
	catch (StopTesting& ex) {
		cancelled = true;
	}

	gettimeofday (&_stopTime, &tz);
	endSession (cancelled);
}


bool TestManager::supportsCancel () const {
	vector<TestObserver*>::const_iterator pivot;
	for (pivot = _activeObservers.begin (); pivot != _activeObservers.end (); ++pivot) {
		if (! (*pivot)->permitsCancel ()) {
			return false;
		}
	}
	return true;
}


bool TestManager::tryCancel () {
	if (supportsCancel ()) {
		_stop = true;
		return true;
	}
	return false;
}


TestPhase TestManager::getCurrentTestPhase () {
	return _execGate._phase;
}


time_t TestManager::getStartTime () const {
	return static_cast<time_t>(_startTime.tv_sec);
}


time_t TestManager::getStopTime () const {
	return static_cast<time_t>(_stopTime.tv_sec);
}


void TestManager::getRunTime (time_t& sec, long& usec) const {
	sec  = static_cast<time_t>(_stopTime.tv_sec - _startTime.tv_sec);
	usec = _stopTime.tv_usec - _startTime.tv_usec;
	if (usec < 0) {
		usec += 1000000;
		sec  -= 1;
	}
}


void TestManager::addObserver (TestObserver* observer) {
	  _testObservers.push_back (observer);
	_activeObservers.push_back (observer);
}


void TestManager::removeObserver (TestObserver* observer) {
	// Saves including algorithms here...
	vector<TestObserver*>::iterator pivot;
	for (pivot = _testObservers.begin (); pivot != _testObservers.end (); ++pivot) {
		if (*pivot == observer) {
			_testObservers.erase (pivot);
			return;
		}
	}
}


void TestManager::configureFrom (ArgVector& args) {
	_activeObservers.clear ();

	vector<TestObserver*>::iterator pivot;
	for (pivot = _testObservers.begin (); pivot != _testObservers.end (); ++pivot) {
		if ((*pivot)->configureFrom (args)) {
			_activeObservers.push_back (*pivot);
		}
	}

}


void TestManager::takeFixture (TestFixture* fixture) {
	  _fixtures.push_back (fixture);
}


//... Private Member Functions


void TestManager::checkForSkippedTests (Test& test) {
	if (_execGate._phase == SET_UP || _execGate._phase == UNKNOWN) {
		if (! test.isTestCase ()) {
			_skipped += test.noTestCases ();
			testSkipped (test);
		}
	}
}


void TestManager::logException (Test* origin, TestException* exception, TestPhase phase) {
	logException (new TestExceptionRecord (origin, exception, phase));
}


void TestManager::logException (TestExceptionRecord* record) {
	if (_recordExceptions) {
		_testExceptions.push_back (record);
	}

	vector<TestObserver*>::iterator pivot;
	for (pivot = _activeObservers.begin (); pivot != _activeObservers.end (); ++pivot) {
		(*pivot)->logException  (*record, *this);
	}
}


void TestManager::beginSession () {
	vector<TestObserver*>::iterator pivot;
	for (pivot = _activeObservers.begin (); pivot != _activeObservers.end (); ++pivot) {
		(*pivot)->beginSession (*this);
	}
}


void TestManager::endSession (bool cancelled) {
	vector<TestObserver*>::iterator pivot;
	for (pivot = _activeObservers.begin (); pivot != _activeObservers.end (); ++pivot) {
		(*pivot)->endSession (*this, cancelled);
	}
}


void TestManager::beginTest (Test& test) {
	vector<TestObserver*>::iterator pivot;
	for (pivot = _activeObservers.begin (); pivot != _activeObservers.end (); ++pivot) {
		(*pivot)->beginTest (test, *this);
	}
}


void TestManager::endTest (Test& test, bool success) {
	vector<TestObserver*>::iterator pivot;
	for (pivot = _activeObservers.begin (); pivot != _activeObservers.end (); ++pivot) {
		(*pivot)->endTest (test, *this, success);
	}
}


void TestManager::testSkipped (Test& test) {
	vector<TestObserver*>::iterator pivot;
	for (pivot = _activeObservers.begin (); pivot != _activeObservers.end (); ++pivot) {
		(*pivot)->testSkipped (test, *this);
	}
}


} // End of namespace Scrunitizer
