/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.4 / Last changed: 30.10.00
*/

#ifndef _TEST_MANAGER_H_
#define _TEST_MANAGER_H_


#include <list>
#include <vector>

#include <Callable.hh>
#include <Test.hh>
#include <TestPhase.hh>
#include <TestConductor.hh>
#include <TestObserver.hh>
#include <TestSession.hh>
#include <TestExceptions.hh>
#include <TestExceptionRecord.hh>
#include <TestFixture.hh>


namespace Scrunitizer {

//: Implementation of the unit test execution logic.
// This class implements the mechanism for executing tests, including
// both test cases and test suites. It isolates the execution of fixture set-up, test
// execution and fixture tear-down operations. That guarantees proper execution of 
// complex hierarchies of test suites and test cases. Each phase of a test execution
// is directed through all registered call gates. Their main purpose is to transform
// domain exceptions into test failures or test errors. Last but not least, all kind of
// states of test execution are reported to all enabled test observers.
//
class TestManager : public TestConductor, public TestSession {

	//: Test manager internal class for controlling the execution of a test.
	// This inner class of test manager actually implements the
	// execution of a test. <I>Not meant to be used or subclassed
	// by test drivers.</I>
	//
	class ExecutionGate : public Callable {

		Test*		_test;
		TestPhase	_phase;
		TestManager*	_manager;

		ExecutionGate (
			TestManager*	manager
		) : _test (0), _phase (UNKNOWN), _manager (manager) {
		}

		virtual void call ();

		friend class TestManager;
	};

	friend class ExecutionGate;


	//: Used to stop a running test session.
	// Used only internally by a test manager.
	//
	struct StopTesting {};


	private:
		ExecutionGate			_execGate;
		Callable*			_baseCallable;
		list<CallGate*>			_callGates;
		int				_executed;
		int				_failed;
		int				_skipped;
		int				_failures;
		int				_errors;
		vector<TestExceptionRecord*>	_testExceptions;
		bool				_recordExceptions;
		vector<TestObserver*>		_testObservers;
		vector<TestObserver*>		_activeObservers;
		bool				_stop;
		mutable struct timeval		_startTime;
		mutable struct timeval		_stopTime;
		list<TestFixture*>		_fixtures;

	public:
		//: Default constructor
		// Sets the internal state so that a test suite could be executed by the manager.
		// if <TT>recordExceptions</TT> is true, all TestException instances are recorded 
		// internally. Most TestObserver implementations rely on the fact that exceptions are
		// recorded by the manager. This feature should only be disabled in special
		// cases when huge amounts of test cases must be executed and special 
		// test observers are used.
		//
		TestManager (bool recordExceptions = true);

		//: Default destructor
		// Deletes all call gates that should be released and 
		// frees all test exception records.
		//
		virtual ~TestManager ();

	private:
		/* Prevent using the copy-constructor and the assignment operator. */
		TestManager (const TestManager&);
		void operator= (const TestManager&);

	public:
		//: Execute a test.
		// This method handles the execution of <I>a single test</I>, most likely a
		// test suite. Used by test case and test suite implementations. External users 
		// of this class should use <TT>conduct</TT> to start a test session.
		//
		virtual bool execute (Test&);

		//: A test is skipped.
		// The internal bookkeeping is updated and the event is logged.
		//
		virtual void skip (Test&);

		//: Adds a call gate.
		// The call gate is deleted when the manager
		// is destroyed.
		//
		virtual void addCallGate (CallGate* gate);

		//: Removes a call gate.
		// The call gate is romoved if it was registered.
		//
		virtual void removeCallGate (CallGate* gate);

		//: Returns the number of test that the manager has encountered do far.
		// The number of encountered test is the number of the executed tests plus
		// the number of test that were skipped.
		// This function is useful for progress reporting.
		//
		virtual int noEncountered () const {
			return _executed + _skipped;
		}

		//: The number of executed tests.
		// Equal to the number of succeeded and failed tests.
		//
		virtual int noExecuted () const {
			return _executed;
		}

		//: The number of test that went fine.
		// Reader method.
		//
		virtual int noPassed () const {
			return _executed - _failed;
		}

		//: The number of test that generated a test failure or a test error.
		// Note that this number is different from what is returned by noFailures and 
		// noErrors.
		//
		virtual int noFailed () const {
			return _failed;
		}

		//: The number of test that were skipped.
		// This is the number of test that were deliberately not executed.
		//
		virtual int noSkipped () const {
			return _skipped;
		}

		//: The number of test failures generated so far.
		// This is the number of test failures, <I>not</I> the number of unsuccessful tests! 
		// (see also noFailed).
		//
		virtual int noFailures () const {
			return _failures;
		}

		//: The number of test errors encountered so far.
		// Note that one test can generate two test errors! One error could 
		// come from a  test action and another one from an erroneous test
		// fixture tear-down.
		// 
		virtual int noErrors () const {
			return _errors;
		}

		//: Returns the all test exceptions generated during execution of a test suite
		// The returned vector should <I>not</I> be altered.
		//
		virtual vector<TestExceptionRecord*>& getTestExceptions () {
			return _testExceptions;
		}

		//: Resets the manager to its initial state.
		// This function must be called when a manager instance is reused
		// for conducting the execution of another test session.
		//
		virtual void reset ();

		//: Conduct the execution of a test session.
		// This operation must be called by the client to execute
		// a test session (a test suite or even only a single test case) under 
		// control of the test manager. 
		//
		virtual void conduct (Test&);

		//: Check if cancelling test sessions is supported.
		// To figure out whether that is possible or not, all active observers are
		// asked if they permit cancel. Only when all active observer permit cancelling
		// test sessions, true is returned.
		//
		virtual bool supportsCancel () const;

		//: Try to cancel executing tests.
		// Invoking this functions makes the manager immediately stop all
		// test activities and return to the caller which invoked the conduct
		// operation. This function should only be called from functions
		// invoked through the test manager (i.e. in <TT>endTest</TT> or <TT>logException</TT>).
		//
		virtual bool tryCancel ();

		//: Inquires the current execution state of the actual test.
		// This function tells the caller in which phase of a test execution the
		// session is currently in.
		//
		TestPhase getCurrentTestPhase ();

		//: Returns the start time of the session.
		// See TestSession for details.
		//
		virtual time_t getStartTime () const;

		//: Returns the session stop time.
		// See TestSession for details.
		//
		virtual time_t getStopTime () const;

		//: Returns the run time of the test session.
		// See TestSession for details.
		//
		virtual void getRunTime (time_t& sec, long& usec) const;

		//: Add a test observer.
		// The observer is added to the set of registered test observers,
		// which is equal to the set to the set of active test observers before
		// function configureFrom is called.
		//
		virtual void addObserver (TestObserver* observer);

		//: Removes a test observers.
		// The observer is removed from the set of registered (and active)
		// test observers.
		//
		virtual void removeObserver (TestObserver* observer);

		//: The test manager and all registered observers are configured.
		// For every registered test observer <TT>configureFrom</TT> is called as well.
		// Only when this returns true, the observer remains active.
		// Note that this functions <I>could modify</I> the contents of <TT>args</TT>!
		// So if the argument vector should be used more than once, a copy must be used here.
		//
		void configureFrom (ArgVector& args);

		//: Take a test fixture.
		// The fixture is owned by the test manager after calling this function,
		// so the fixture is deleteled when the manager is deleted.
		//
		virtual void takeFixture (TestFixture* fixture);

	private:
		bool executePhase (Test& test, TestPhase phase);
		void checkForSkippedTests (Test& test);
		void logException (Test* origin, TestException* exception, TestPhase phase);
		void logException (TestExceptionRecord* record);
		void beginSession ();
		void endSession (bool cancelled);
		void beginTest (Test& test);
		void endTest (Test& test, bool success);
		void testSkipped (Test& test);

};


} // End of namespace Scrunitizer

#endif
