/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 30.07.00
*/

#ifndef _TEXT_OBSERVER_H_
#define _TEXT_OBSERVER_H_


#include <TestSession.hh>
#include <Options.hh>


namespace Scrunitizer {

//: Interface for test observers.
// A test observer is an object that supervises the execution of unit tests.
// It is informed by the TestManager about the execution of unit
// tests and about all generated exceptions. A test observer instance must register itself
// to the TestManager before tests are executed.
//
class TestObserver {

	protected:
		//: Virtual destructor.
		// Ensures that all derived classes have a virtual destructor too.
		//
		virtual ~TestObserver () {}

	public:
		//: Configure the observer from command line options.
		// This functions allows the observer to configure itself
		// according to the given command line options (in the usual 
		// C/C++ command line format). When this function returns true,
		// The observer is included in observing the execution of tests.
		// If false is returned, the observer is disabled until this
		// function called again.
		//
		virtual bool configureFrom (ArgVector& args) {
			return true;
		}

		//: Check if the observer permits cancelling the execution of tests.
		// If this function returns true, the TestManager is allowed to cancel
		// a test session; it does <I>not</I>mean that it <I>will</I> stop
		// all test activities. If another observer returns false, the TestManager
		// will continue testing.
		//
		virtual bool permitsCancel () const {
			return true;
		}

		//: A new test session is started.
		// This function is called by the TestManager before it starts running
		// unit tests. The observer could do necessary initialization steps
		// and/or it could generate a test report header.
		//
		virtual void beginSession (TestSession& session) = 0;

		//: A test session has come to an end.
		// After the test session was terminated, this function is called.
		// Parameter <TT>cancelled</TT> is true when the session was stopped during execution.
		// In this case, most likely not all tests have been executed.
		//
		virtual void endSession (TestSession& session, bool cancelled) = 0;

		//: Another test is executed.
		// This functions is called before another unit test is run. It does not
		// matter if that test is a test case or a test suite.
		//
		virtual void beginTest (Test& test, TestSession& session) = 0;

		//: A test was run.
		// Called after running the test was finished. The parameter <TT>success</TT> is
		// true when no test exception was thrown during execution.
		//
		virtual void endTest   (Test& test, TestSession& session, bool success) = 0;

		//: A test is skipped.
		// When a test is <I>directly</I> skipped, this function is called. It is <I>not</I>
		// called for test cases and test suites contained in a test suite that
		// is skipped itself. That also means that the test object could be a test case
		// or a test suite.
		//
		virtual void testSkipped (Test& test, TestSession& session) = 0;

		//: A test exception has occurred.
		// If some test exception was generated by a test, this function is called.
		//
		virtual void logException (TestExceptionRecord& mr, TestSession& session) = 0;
};



} // End of namespace Scrunitizer

#endif

