/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 05.08.00
*/

#ifndef _TEST_REPEATER_H_
#define _TEST_REPEATER_H_


#include <Test.hh>


namespace Scrunitizer {


//: Repeats tests multiple times
// This class could be used to repeat unit test or even whole
// test suites a given number of times. This also includes
// the repeated execution of the set-up and tear-down methods.
//
class TestRepeater : public Test {

	private:
		Test*		_test;
		int		_repetitions;
		bool		_breakOnFailure;
		bool		_success;

	public:
		//: Create a test repeater with the given name and test to repeat.
		// The given test is executed the specified number of repetitions.
		// If <TT>breakOnFailure</TT> is true, repeating the test is stopped
		// immediately when the test fails. Otherwise, all repetitions are executed
		// regardless if executing the test is successful or not.
		//
		TestRepeater (const string& name, int repetitions, Test* test, bool breakOnFailure = false);

		//: Destructor for the test repetition
		// The given base test is also deleted.
		//
		virtual ~TestRepeater ();

	public:
		//: Run the base test n-times under control of then given test conductor.
		// The base test specified in the constructor is executed multiple
		// times under control of the conductor.
		//
		virtual bool run (TestConductor& conductor);

		//: Set up the test environment.
		// Does nothing.
		//
		virtual void setUp () {}

		//: Run the base test multiple times.
		// How often the test is actually executed the full number of repetitions
		// depends on <TT>breakOnFailure</TT> and the test outcomes (see constructor).
		//
		virtual void test (TestConductor& conductor);

		//: Tear down the test environment.
		// Does nothing.
		//
		virtual void tearDown () {}

		//: Test if this is a test case.
		// Returns always false, because <I>the repeater itself
		// does not check functionality in the application domain</I>.
		// So it is more similar to a test suite.
		//
		virtual bool isTestCase () const {
			return false;
		};

		//: Count the number of test cases.
		// Returns the number of test cases in the base test, multiplied
		// with the number of repetitions.
		//
		virtual int noTestCases () const;

		//: Count the number of test suites.
		// Returns the number of test suites in the base test, multiplied
		// with the number of repetitions plus one.
		//
		virtual int noTestSuites () const;

		//: Output the test repeater to the stream.
		// The name and the number of repetitions are printed.
		//
		virtual void put (ostream& stream) const {
			stream << "Test repeater: " << getName () << " {" << _repetitions << "}";
		}
};


} // End of namespace Scrunitizer

#endif
