/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 30.07.00
*/

#ifndef _TEST_SESSION_H_
#define _TEST_SESSION_H_


#include <sys/time.h>
#include <unistd.h>
#include <vector>

#include <TestExceptionRecord.hh>


namespace Scrunitizer {


//: Interface representing the execution of a set of unit tests.
// A test session is the observers viewpoint on the execution of a set
// of unit tests.
// It offers mostly query functions for the state of a test session
// and an operation for trying to cancel an ongoing session.
// This interface is used while interacting with instances of TestObserver.
// It is implemented by TestManager.
//
class TestSession {

	public:
		//: Destructor.
		// Enforces that derived classes have also a virtual constructor.
		//
		virtual ~TestSession () {}

	public:
		//: Returns the number of tests that were encountered in this session so far.
		// The number of encountered test is the number of the executed tests plus
		// the number of test that were skipped.
		// Knowing number of encountered tests is particularly useful for progress reporting.
		//
		virtual int noEncountered () const = 0;

		//: The number of executed tests.
		// Equal to the number of succeeded and failed tests.
		//
		virtual int noExecuted () const = 0;

		//: The number of test that went fine.
		// A test has passed successfully if it has not generated any exceptions.
		//
		virtual int noPassed () const = 0;

		//: The number of test that generated a test failure or a test error.
		// Note that this number is different from what is returned by noFailures and 
		// noErrors.
		//
		virtual int noFailed () const = 0;

		//: The number of test that were skipped.
		// This is the number of test that were deliberately not executed.
		//
		virtual int noSkipped () const = 0;

		//: The number of test failures generated so far.
		// This is the number of test failures, <I>not</I> the number of unsuccessful tests! 
		// (see also <TT>noFailed</TT>).
		//
		virtual int noFailures () const = 0;

		//: The number of test errors encountered so far.
		// Note that one test can generate two test errors! One error could 
		// come from a test action and another one from an erroneous test
		// fixture tear-down.
		// 
		virtual int noErrors () const = 0;

		//: Returns the all test exceptions generated in this session so far.
		// The returned vector must <I>not</I> be modified!
		//
		virtual vector<TestExceptionRecord*>& getTestExceptions () = 0;

		//: Check if cancelling a test session is possible.
		// This function returns true when cancelling a test session is possible.
		// It does <I>not</I> mean that calling cancel will be allways successful.
		// I.e., this functions could be used when deciding if a stop button
		// could be enabled or not.
		//
		virtual bool supportsCancel () const = 0;

		//: Try to cancel the test session.
		// With this function it could be tried to cancel an ongoing test session.
		// The actual test performer could decide if it wants to cancel the session
		// or continue running the session. In case the session is cancelled, true
		// is returned. Calling this function only makes sense if <TT>supportsCancel</TT>
		//  returns true.
		//
		virtual bool tryCancel () = 0;

		//: Inquires the current execution state of the actual test.
		// This function tells the caller in which phase of a test execution the
		// session is currently in.
		//
		virtual TestPhase getCurrentTestPhase () = 0;

		//: Returns the start time of the session.
		// The format of the return value is seconds since epoch, 01.01.1970, 00:00.
		// The returned value is a "native OS time values", meaning that the time
		// base (UTC or localised time) depends on the OS settings.
		//
		virtual time_t getStartTime () const = 0;

		//: Returns the session stop time.
		// See getStartTime for information about the time format and base.
		//
		virtual time_t getStopTime () const = 0;

		//: Returns the run time of the test session.
		// This is the real time clock difference between start and stop time.
		// The parameter <TT>usec</TT> enhances the resolution up to microseconds.
		// The real resolution of the <TT>usec</TT> value depends on the OS and is
		// most likely considerably greater than one microsecond.
		//
		virtual void getRunTime (time_t& sec, long& usec) const = 0;

};


} // End of namespace Scrunitizer

#endif
