/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 19.08.00
*/


#ifndef _TEST_SUITE_H_
#define _TEST_SUITE_H_


#include <vector>

#include <Test.hh>


namespace Scrunitizer {

//: Base implementation for test suites.
// This class forms the basic skeleton for test suites. It could be used as base
// class for real test suite implementations. A test suite is 
// a set of unit tests (instances which implement the Test interface) that
// should be executed altogether. Normally this is the case when all members 
// of a suite test functionality in the same (sub)domain. A test suite could
// contain test cases and other test suites. This allows to structure 
// unit tests in a hierarchical, file-system like manner.
//
class TestSuite : public Test {

	protected:
		//: All contained tests.
		// The contained tests in the order in which they were inserted.
		//
		vector<Test*>	_tests;

		//: Break on failure flag.
		// When true, executing the suite is stopped immediately when
		// one of the contained tests fails.
		//
		bool		_breakOnFailure;

		//: Success status of the suite.
		// Signals the final outcome of the suite.
		// Altered during execution of the suite. Initially true,
		// goes to false as soon as one test fails.
		//
		bool		_success;

	public:
		//: Create a test suite.
		// Sets the suites name. If <TT>breakOnFailure</TT> is true, the suite
		// stops executing the remaining tests if a test fails. Otherwise, always
		// all tests are executed. 
		//
		TestSuite (const string& name, bool breakOnFailure = false);

		//: The destructor.
		// Also all contained tests are deleted.
		//
		virtual ~TestSuite ();

	private:
		/* Prevent using the copy-constructor and the assignment operator. */
		TestSuite (const TestSuite&);
		void operator= (const TestSuite&);

	public:
		//: Make the conductor run the test suite.
		// Executing the test suite and returning the success state is 
		// delegated to the conductor.
		//
		virtual bool run (TestConductor&);


		//: Set up the test suite
		// Dummy implementation with no functionality.
		// If all members of the test suite need the same test environment
		// which need not to be restored before each test, the installation
		// of the shared test environment could be done in this method.
		//
		virtual void setUp () {}

		//: Run the tests in the suite
		// All members of the test suite are run under the control
		// of the given test conductor.
		//
		virtual void test (TestConductor&);

		//: Tear down the test environment of the suite.
		// Dummy implementation with no functionality.
		//
		virtual void tearDown () {}

		//: Test if the suite is a test case.
		// Always returns false.
		//
		virtual bool isTestCase () const;

		//: Count the number of test cases
		// Computes the number of test cases in the suite and all
		// contained sub-suites.
		//
		virtual int noTestCases () const;

		//: Counts the number of test suites in the suite
		// Returns the number of (directly or indirectly) contained
		// test suites plus one.
		//
		virtual int noTestSuites () const;

		//: Output the test suite to the stream.
		// The string "Test suite: " and the name is printed.
		// 
		virtual void put (ostream& stream) const {
			stream << "Test suite: " << getName ();
		}

		//: Add a test
		// This operation is used to add test cases and other sub-suites to the
		// test suite. It is not checked if this test is already contained in the
		// suite.
		//
		virtual void add (
			Test* test
		);

};


} // End of namespace Scrunitizer

#endif
