/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 05.08.00
*/


#ifndef _TEST_SUITE_ADAPTER_H_
#define _TEST_SUITE_ADAPTER_H_


#include <TestSuite.hh>
#include <Binder.hh>


namespace Scrunitizer {


//: Creates test suites that use functors for fixture set-up and tear-down.
// This template creates a test suite class that calls functors during
// set-up and tear-down of the test suite. Instead of instantiating the template 
// directly, the convenience function <TT>makeTestSuite</TT> could be used:
// <P>
// <TT><PRE>
//template&lt;typename S, typename D&gt; inline TestSuiteAdapter&lt;S,D&gt;* makeTestSuite (
//      const string&amp; name,
//      bool breakOnFailure,
//      S setUpFunc,
//      D tearDownFunc
//);
//
//template&lt;typename S&gt; inline TestSuiteAdapter&lt;S,noop_type&gt;* makeTestSuite (
//      const string&amp; name,
//      bool breakOnFailure,
//      S setUpFunc
//);
// </PRE></TT>
//
template<typename S, typename D> class TestSuiteAdapter : public TestSuite {

	private:
		S	_setUpFunc;
		D	_tearDownFunc;

	public:
		//: Creates a new test suite adapter instance.
		// Beside the name and the break-on-failure status, a functor for
		// set-up (of type S) and tear-down (of type D) is needed.
		// See TestSuite for the meaning of <TT>breakOnFailure</TT>.
		//
		TestSuiteAdapter(const string name,
				 bool breakOnFailure,
				 S setUpFunc,
				 D tearDownFunc
		) : 	TestSuite (name, breakOnFailure), 
			_setUpFunc (setUpFunc), 
			_tearDownFunc (tearDownFunc) {
		}

	public:
		//: Test suite set-up.
		// The set-up functor of type S is called.
		//
		virtual void setUp () {
			_setUpFunc ();
		}

		//: Test suite tear-down.
		// The tear-down functor of type D is called.
		//
		virtual void tearDown () {
			_tearDownFunc ();
		}

};


template<typename S, typename D> inline TestSuiteAdapter<S,D>* makeTestSuite (
	const string& name,
	bool breakOnFailure,
	S setUpFunc,
	D tearDownFunc
) {
	return new TestSuiteAdapter<S,D> (name, breakOnFailure, setUpFunc, tearDownFunc);
}


template<typename S> inline TestSuiteAdapter<S,noop_type>* makeTestSuite (
	const string& name,
	bool breakOnFailure,
	S setUpFunc
) {
	return new TestSuiteAdapter<S,noop_type> (name, breakOnFailure, setUpFunc, noop);
}


} // End of namespace Scrunitizer
#endif

