/* -*- Mode: Fundamental -*-
   
   This file is part of Scrunitizer - the C++ Unit Test Framework
   Copyright (C) 2000  Bernd Linowski
   (scrunitizer@linosphere.de)
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or any later version.
   
   This framework is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   

   Version 0.3 / Last changed: 09.08.00
*/

#ifndef _TEST_WRAPPER_H_
#define _TEST_WRAPPER_H_


#include <Test.hh>


namespace Scrunitizer {


//: Wrapper for test objects.
// Objects of this class delegate all operations to
// the shadowed test object. 
// This class serves as base class for other classes
// realizing the proxy and decorator pattern.
//
class TestWrapper : public Test {

	private:
		Test*	_test;
		bool	_release;

	protected:
		//: Create a test wrapper for the given test object.
		// The wrapper shadows the test object. 
		// By default, it is assumed that the shadowed test is owned
		// by another object (<TT>release</TT> is false). But if <TT>release</TT> is true,
		// the shadowed test is deleted when the wrapper is destroyed.
		//
		TestWrapper (
			const string& name,
			Test* test, 
			bool release
		) : Test (name), _test (test), _release (release) {
			if (test == 0) {
				throw IllegalTestSetUp ();
			}
		}

		//: Create a test wrapper for the given test object.
		// Like above, but with no specific name; "Wrapper" is used instead.
		//
		TestWrapper (
			Test* test, 
			bool release = false
		) : Test ("Wrapper"), _test (test), _release (release) {
			if (test == 0) {
				throw IllegalTestSetUp ();
			}
		}

	public:
		//: Destructor.
		// Deletes also the shadowed test object if <TT>release</TT> was set to true.
		//
		virtual ~TestWrapper () {
			if (_release) delete _test;
		}

	public:
		//: Returns the name of the shadowed test.
		// See interface Test for details.
		//
		virtual string getName () const {
			return _test->getName ();
		}

		//: Delegate the execution of the shadowed test to the given test conductor.
		// See interface Test for details.
		//
		virtual bool run (TestConductor& conductor) {
			return _test->run (conductor);
		}

		//: Set-up the test fixture.
		// See interface Test for details.
		//
		virtual void setUp () {
			_test->setUp ();
		}

		//: Execute the shadowed test.
		// See interface Test for details.
		//
		virtual void test (TestConductor& conductor) {
			_test->test (conductor);
		}

		//: Tear-down of the test fixture.
		// See interface Test for details.
		//
		virtual void tearDown () {
			_test->tearDown ();
		}

		//: Check if the shadowed test is a test case.
		// See interface Test for details.
		//
		virtual bool isTestCase () const {
			return _test->isTestCase ();
		}

		//: Returns the number of contained test cases in the shadowed test.
		// See interface Test for details.
		//
		virtual int noTestCases () const {
			return _test->noTestCases ();
		}

		//: Returns the number of contained test suites in the shadowed test.
		// See interface Test for details.
		//
		virtual int noTestSuites () const {
			return _test->noTestCases ();
		}

		//: Output the test case to the stream.
		// Prefixes the output of the shadowed test by stating that this instance is 
		// a wrapper (or owner-wrapper) of the test.
		//
		virtual void put (ostream& stream) const {
			stream << ((_release) ? "Wrapper" : "Owner-wrapper") << " of ";
			_test->put (stream);
		}


		//: Returns the shadowed test.
		// Reader function.
		//
		virtual Test* getTestObject () const {
			return _test;
		}

		//: Sets the shadowed test.
		// In addition, the release status could be specified.
		// (See also the constructor).
		//
		virtual void setTestObject (
			Test* test,
			bool release = true
		) {
			if (test == 0) throw IllegalTestSetUp ();

			if (_release) delete _test;

			_test	 = test;
			_release = release;
		}
};


} // End of namespace Scrunitizer

#endif
